#include "ConfirmationPage.h"
#include "mylib/wtlext/EditCtrlStream.h"
#include "core/SourceSetting.h"
#include "core/SlipstreamSetting.h"
#include "core/OutputSetting.h"

//ConfirmationPage::ConfirmationPage()
//{
//}
//
//ConfirmationPage::~ConfirmationPage()
//{
//}

void ConfirmationPage::loadSettings(SourceSetting const *source_setting,
                                    SlipstreamSetting const *slipstream_setting,
                                    OutputSetting const *output_setting)
{
	// edit control S
	this->edit1.SetWindowText(L"");

	MyLib::EditCtrlStream edit_ctrl(this->edit1.m_hWnd);

	edit_ctrl << _T("^[QbgF\n\t");
	uint32_t const os_ver = source_setting->getOSVersion();
	int const product_type = source_setting->getProductType();
	int const architecture_id = source_setting->getArchitectureID();
	switch (os_ver) {
	case 0x00050000:
		edit_ctrl << _T("Windows 2000");
		if (product_type == 0) {
			edit_ctrl << L" Professional";
		} else if (product_type == 1) {
			edit_ctrl << L" Server";
		}
		break;
	case 0x00050001:
		edit_ctrl << _T("Windows XP");
		if (product_type == 0) {
			edit_ctrl << L" Professional";
		} else if (product_type == 4) {
			edit_ctrl << L" Home Edition";
		}
		break;
	case 0x00050002:
		if (product_type == 0) {
			edit_ctrl << L"Windows XP Professional";
		} else {
			edit_ctrl << _T("Windows Server 2003");
			if (product_type == 1) {
				edit_ctrl << L" Standard Edition";
			} else if (product_type == 2) {
				edit_ctrl << L" Enterprise Edition";
			} else if (product_type == 3) {
				edit_ctrl << L" Datacenter Edition";
			}
		}
		break;
	default:
		edit_ctrl << _T("sOS");
		break;
	}
	switch (architecture_id) {
	case PROCESSOR_ARCHITECTURE_AMD64:
		edit_ctrl << _T(" x64");
		break;
	case PROCESSOR_ARCHITECTURE_IA64:
		edit_ctrl << _T(" for Itanium-based systems");
		break;
	}
	int const current_sp_num = source_setting->getCurrentSPNumber();
	if (current_sp_num == 0) {
		edit_ctrl << L" ";
	} else {
		edit_ctrl << L" SP" << current_sp_num;
	}

	edit_ctrl << _T("\n") << std::endl;

	edit_ctrl << _T("CXg[t@C̏ꏊF") << std::endl;
	type_info const &t_info = typeid(*source_setting);
	if (t_info == typeid(DirectorySourceSetting) || t_info == typeid(DiscSourceSetting)) {
		DirectorySourceSetting const *ptr = static_cast<DirectorySourceSetting const*>(source_setting);
		edit_ctrl << L"\t" << ptr->getSourcePath().native_directory_string() << L"\n" << std::endl;
	}

	std::wstring const work_dir_path = slipstream_setting->getWorkDirPath().native_directory_string();
	edit_ctrl << L"ƃtH_F\n\t"
	          << (work_dir_path.empty() ? L"ǂݍݐƓ" : work_dir_path) << L'\n';
	if (slipstream_setting->createsTempWorkDir()) {
		edit_ctrl << L"\tꎞtH_𗘗p\n";
	}
	edit_ctrl << std::endl;

	std::wstring const sp_file_path = slipstream_setting->getSPFileName().native_file_string();
	edit_ctrl << L"T[rXpbNt@CF\n\t"
	          << (sp_file_path.empty() ? L"Kpς݂ɂȂ" : sp_file_path) << L'\n'
	          << std::endl;

	// BigDrive
	if (os_ver == 0x00050000) {
		edit_ctrl << L"137GBȏ̃n[hfBXNF\n\t"
		          << (slipstream_setting->usesBigdriveW2K() ? L"gp\n" : L"gpȂ\n")
		          << std::endl;
	}
	// Clean Install Only
	edit_ctrl << L"N[CXg[ɕKv̂Ȃt@C̍폜F\n\t"
	          << (slipstream_setting->isCleanInstallOnly() ? L"\n" : L"Ȃ\n")
	          << std::endl;
	// Recompresses Driver.cab
	edit_ctrl << L"WhCoCABt@C܂Ƃ߂čkňkȂF\n\t"
	          << (slipstream_setting->executesDriverCabRecompress() ? L"\n" : L"Ȃ\n")
	          << std::endl;

	edit_ctrl << L"hCoF\n";
	std::vector<DriverSetupData> const &drivers = slipstream_setting->getAddDriverCollection();
	typedef std::vector<DriverSetupData>::const_iterator DriverDataIterator;
	typedef DriverSetupData::DDInstallCollection DDInstallCollection;
	DriverDataIterator it = drivers.begin();
	if (it == drivers.end()) {
		edit_ctrl << L"\tȂ\n";
	} else {
		for (; it != drivers.end(); ++it) {
			edit_ctrl << L'\t';
			DDInstallCollection const &ddic = it->getDDInstallSectionCollection();
			DDInstallCollection::const_iterator it2 = ddic.begin();
			if (it2 != ddic.end()) {
				DriverSetupData::DescriptionCollection::const_iterator it3 =
					it2->second.descriptions.begin();
				if (it3 != it2->second.descriptions.end()) {
					edit_ctrl << *it3;
				}
			}
			edit_ctrl << L"\n\t\t(" << it->getInfFilePath().native_file_string() << L")\n";
		}
	}
	edit_ctrl << std::endl;

	std::wstring const iso_image_path = output_setting->getISOImagePath().native_file_string();
	//const std::wstring iso_image_path = this->integration_setting.getISOImagePath().str();
	std::wstring const record_device_str = output_setting->getRecordDevice();
	//const std::wstring record_device_str = this->integration_setting.getRecordDeviceString();
	if (!iso_image_path.empty() || record_device_str.empty()) {
		edit_ctrl << _T("{[xF") << std::endl;
		std::wstring const volume_label = output_setting->getVolumeLabel();
	//	const std::wstring volume_label = this->integration_setting.getVolumeLabel();
		edit_ctrl << _T("\t") << (volume_label.empty() ? _T("Ȃ") : volume_label) << _T("\n") << std::endl;
	}

	edit_ctrl << L"ISOC[WF\n\t"
	          << (iso_image_path.empty() ? L"쐬Ȃ" : iso_image_path) << L'\n'
	          << std::endl;

	edit_ctrl << L"CDEDVDւ̏݁F\n";
	if (record_device_str.empty()) {
		edit_ctrl << L"\t݂Ȃ\n";
	} else {
		edit_ctrl << L"\t݂ɎgphCu\n\t\t"
		          << record_device_str
		          << L"\n\tݑxF" << output_setting->getRecordSpeed() << L"{\n";
		if (output_setting->erasesMedia()) {
			edit_ctrl << L"\tRWfBXNĂ珑\n";
		}
		if (output_setting->ejectsMedia()) {
			edit_ctrl << L"\t݌AfBXNo\n";
		}
	}
	edit_ctrl << std::endl;

	wchar_t const usb_target_drive = output_setting->getUSBTargetDrive();
	wchar_t const usb_setup_drive = output_setting->getUSBSetupDrive();
	edit_ctrl << L"USBEn[hfBXNȂǂCXg[F" << std::endl;
	if (::iswalpha(usb_target_drive) && ::iswalpha(usb_setup_drive)) {
		edit_ctrl << L"\tgphCu " << usb_target_drive << L":\n"
		          << L"\tCXg[Ɋ蓖ĂhCu " << usb_setup_drive << L":\n";
	} else {
		edit_ctrl << L"\tgpȂ\n";
	}
	edit_ctrl << std::endl;

	//// y[W\ꂽƂAGfBbgRg[SIԂɂȂ̂hB
	//this->is_first_focus = true;

	//::GetDlgItemText(this->GetParent(), ID_WIZNEXT, next_button_caption, _countof(next_button_caption));
	//::SetDlgItemText(this->GetParent(), ID_WIZNEXT, _T("sI"));
	//// http://support.microsoft.com/kb/148388/
	//this->PostMessage(UWM_SET_PAGE_FOCUS);
}

BOOL ConfirmationPage::OnInitDialog(HWND /*wndFocus*/, LPARAM /*lInitParam*/)
{
#ifdef MY_DEBUG
	::WritePrivateProfileString(L"Debug", L"State", L"ConfirmationPage::OnInitDialog() start", PROFILE_PATH->string().c_str());
#endif
	this->edit1 = this->GetDlgItem(IDC_EDIT1);
	this->DlgResize_Init(false);
#ifdef MY_DEBUG
	::WritePrivateProfileString(L"Debug", L"State", L"ConfirmationPage::OnInitDialog() end", PROFILE_PATH->string().c_str());
#endif
	return true;
}
