// F6ToolView.cpp : implementation of the CF6ToolView class
//
/////////////////////////////////////////////////////////////////////////////

#include "F6ToolView.h"
#include "core/SourceSetting.h"
#include "core/SlipstreamSetting.h"
#include "core/OutputSetting.h"

namespace {
	std::wstring original_window_title;
};

CF6ToolView::CF6ToolView()
: m_hWndFocus(NULL)
, active_page_index(0)
{
}

BOOL CF6ToolView::PreTranslateMessage(MSG* pMsg)
{
	return CWindow::IsDialogMessage(pMsg);
	//if (CWindow::IsDialogMessage(pMsg)) {
	//	return TRUE;
	//}
	//return this->work_dir_setting_page.PreTranslateMessage(pMsg);
}

BOOL CF6ToolView::OnInitDialog(HWND /*wndFocus*/, LPARAM /*lInitParam*/)
{
#ifdef MY_DEBUG
	::WritePrivateProfileString(L"Debug", L"State", L"CF6ToolView::OnInitDialog() start", PROFILE_PATH->string().c_str());
#endif

	this->source_setting_page.Create(this->m_hWnd);
	this->slipstream_setting_page.Create(this->m_hWnd);
	this->output_setting_page.Create(this->m_hWnd);
	this->confirmation_page.Create(this->m_hWnd);
	this->progress_page.Create(this->m_hWnd);

	this->source_setting_page.SetDlgCtrlID(IDD_SOURCE_SETTING_PAGE);
	this->slipstream_setting_page.SetDlgCtrlID(IDD_SLIPSTREAM_SETTING_PAGE);
	this->output_setting_page.SetDlgCtrlID(150);// IDD_OUTPUT_SETTING_PAGE == IDD_SLIPSTREAM_SETTING_PAGE
	this->confirmation_page.SetDlgCtrlID(IDD_CONFIRMATION_PAGE);
	this->progress_page.SetDlgCtrlID(IDD_PROGRESS_PAGE);

	this->source_setting_page.ShowWindow(SW_SHOW);
	this->slipstream_setting_page.ShowWindow(SW_HIDE);
	this->output_setting_page.ShowWindow(SW_HIDE);
	this->confirmation_page.ShowWindow(SW_HIDE);
	this->progress_page.ShowWindow(SW_HIDE);

	this->prev_button = this->GetDlgItem(IDC_BUTTON_PREV);
	this->next_button = this->GetDlgItem(IDC_BUTTON_NEXT);
	this->finish_button = this->GetDlgItem(IDC_BUTTON_FINISH);
	this->prev_button.SetWindowPos(HWND_BOTTOM, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
	this->next_button.SetWindowPos(HWND_BOTTOM, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
	this->finish_button.SetWindowPos(HWND_BOTTOM, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
	this->finish_button.EnableWindow(false);
	this->finish_button.ShowWindow(SW_HIDE);


	// EBhE^Cg̕ۑ
	HWND hwnd_parent = this->GetParent();
	int const title_len = ::GetWindowTextLength(hwnd_parent) + 1;
	boost::scoped_array<wchar_t> title_buf(new wchar_t[title_len]);
	::GetWindowText(hwnd_parent, title_buf.get(), title_len);
	original_window_title = title_buf.get();

	std::wstring new_title = original_window_title + L" (1/5)";
	::SetWindowText(hwnd_parent, new_title.c_str());

	this->DlgResize_Init(false);

#ifdef MY_DEBUG
	::WritePrivateProfileString(L"Debug", L"State", L"CF6ToolView::OnInitDialog() end", PROFILE_PATH->string().c_str());
#endif
	return TRUE;
}

void CF6ToolView::OnSetFocus(HWND /*wndOld*/)
{
	if (!::IsWindow(m_hWndFocus) || !::IsChild(m_hWnd, m_hWndFocus)) {
		// invalid or unknown focus window... let windows handle it
		m_hWndFocus = NULL;
		//Default();
		SetMsgHandled(false);
		return;
	}
	// otherwise, set focus to the last known focus window
	::SetFocus(m_hWndFocus);
}

void CF6ToolView::OnActivate(unsigned int /*nState*/, BOOL /*bMinimized*/, HWND /*wndOther*/)
{
	if (SaveFocusControl()) {
		return;     // don't call base class when focus is already set
	}
	SetMsgHandled(false);
}

BOOL CF6ToolView::SaveFocusControl()
{
	// save focus window if focus is on this window's controls
	HWND hWndFocus = ::GetFocus();
	if (hWndFocus != NULL && ::IsChild(m_hWnd, hWndFocus)) {
		m_hWndFocus = hWndFocus;
		return TRUE;
	}
	return FALSE;
}

void CF6ToolView::OnButtonPrev(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	switch (this->active_page_index) {
	case 1:
		this->slipstream_setting_page.ShowWindow(SW_HIDE);
		this->source_setting_page.ShowWindow(SW_SHOW);
		--(this->active_page_index);
		break;
	case 2:
		this->output_setting_page.ShowWindow(SW_HIDE);
		this->slipstream_setting_page.ShowWindow(SW_SHOW);
		--(this->active_page_index);
		break;
	case 3:
		this->confirmation_page.ShowWindow(SW_HIDE);
		this->output_setting_page.ShowWindow(SW_SHOW);
		--(this->active_page_index);
		break;
	case 4:
		this->progress_page.ShowWindow(SW_HIDE);
		this->confirmation_page.ShowWindow(SW_SHOW);
		this->next_button.ShowWindow(SW_SHOW);
		this->finish_button.ShowWindow(SW_HIDE);
		--(this->active_page_index);
		break;
	default:
		break;
	}

	HWND hwnd_parent = this->GetParent();
	wchar_t buf[] = L" (1/5)";
	buf[2] += static_cast<wchar_t>(this->active_page_index);
	std::wstring new_title = original_window_title + buf;
	::SetWindowText(hwnd_parent, new_title.c_str());
}

void CF6ToolView::OnButtonNext(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	static boost::scoped_ptr<SourceSetting> source_setting;
	static boost::scoped_ptr<SlipstreamSetting> slipstream_setting;
	static boost::scoped_ptr<OutputSetting> output_setting;

	switch (this->active_page_index) {
	case 0:
		source_setting.reset(this->source_setting_page.getSetting());
		if (!!source_setting) {
			this->slipstream_setting_page.loadSourceSetting(source_setting.get());
			this->source_setting_page.ShowWindow(SW_HIDE);
			this->slipstream_setting_page.ShowWindow(SW_SHOW);
			++(this->active_page_index);
		}
		break;
	case 1:
		slipstream_setting.reset(this->slipstream_setting_page.getSetting());
		if (!!slipstream_setting) {
			this->output_setting_page.loadSourceSetting(source_setting.get());
			this->slipstream_setting_page.ShowWindow(SW_HIDE);
			this->output_setting_page.ShowWindow(SW_SHOW);
			++(this->active_page_index);
		}
		break;
	case 2:
		output_setting.reset(this->output_setting_page.getSetting());
		if (!!output_setting) {
			this->confirmation_page.loadSettings(source_setting.get(),
			                                     slipstream_setting.get(),
			                                     output_setting.get());
			this->output_setting_page.ShowWindow(SW_HIDE);
			this->confirmation_page.ShowWindow(SW_SHOW);
			++(this->active_page_index);
		}
		break;
	case 3:
		this->progress_page.loadSettings(source_setting.get(),
		                                 slipstream_setting.get(),
		                                 output_setting.get());
		this->confirmation_page.ShowWindow(SW_HIDE);
		this->progress_page.ShowWindow(SW_SHOW);
		this->prev_button.EnableWindow(false);
		this->next_button.ShowWindow(SW_HIDE);
		this->finish_button.ShowWindow(SW_SHOW);
		::SendMessage(this->GetParent(), UWM_DISABLE_EXIT, 0, 0);
		++(this->active_page_index);
		break;
	default:
		break;
	}

	HWND hwnd_parent = this->GetParent();
	wchar_t buf[] = L" (1/5)";
	buf[2] += static_cast<wchar_t>(this->active_page_index);
	std::wstring new_title = original_window_title + buf;
	::SetWindowText(hwnd_parent, new_title.c_str());
}

void CF6ToolView::OnButtonFinish(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	::PostMessage(this->GetParent(), WM_CLOSE, 0, 0);
}

LRESULT CF6ToolView::OnEnableFinishButton(unsigned int /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	::SendMessage(this->GetParent(), UWM_ENABLE_EXIT, 0, 0);
	return this->finish_button.EnableWindow();
}

LRESULT CF6ToolView::OnEnableBackButton(unsigned int /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	::SendMessage(this->GetParent(), UWM_ENABLE_EXIT, 0, 0);
	return this->prev_button.EnableWindow();
}

LRESULT CF6ToolView::OnPopupMenuSelect(unsigned int /*uMsg*/, WPARAM wParam, LPARAM lParam)
{
	return this->GetParent().PostMessage(WM_MENUSELECT, wParam, lParam);
}

//LRESULT CF6ToolView::OnDriverListImport(unsigned int, WPARAM, LPARAM)
//{
//	return this->slipstream_setting_page.SendMessage(DriverSettingPage::UWM_DRIVER_LIST_IMPORT);
//}
//
//LRESULT CF6ToolView::OnDriverListExport(unsigned int, WPARAM, LPARAM)
//{
//	return this->slipstream_setting_page.SendMessage(DriverSettingPage::UWM_DRIVER_LIST_EXPORT);
//}
