#include "ProgressPage.h"
#include "core/WorkManager.h"
#include "mylib/wtlext/EditCtrlStream.h"
#include "mylib/function.h"

ProgressPage::ProgressPage()
: source_setting(NULL)
, slipstream_setting(NULL)
, output_setting(NULL)
{
}

ProgressPage::~ProgressPage()
{
}

void ProgressPage::eventHappened(std::wstring const &arg)
{
	MyLib::EditCtrlStream edit_ctrl(this->edit1.m_hWnd);
	edit_ctrl << arg << std::flush;
}

void ProgressPage::loadSettings(SourceSetting const *source_setting,
                                SlipstreamSetting const *slipstream_setting,
                                OutputSetting const *output_setting)
{
	this->source_setting = source_setting;
	this->slipstream_setting = slipstream_setting;
	this->output_setting = output_setting;
	this->edit1.SetWindowText(L"");
	::_beginthreadex(NULL, 0, threadMain, this, 0, NULL);
}

unsigned int WINAPI ProgressPage::threadMain(void *param)
{
	ProgressPage *instance = reinterpret_cast<ProgressPage*>(param);
	assert(instance->IsWindow());
	MyLib::EditCtrlStream edit_ctrl(instance->edit1.m_hWnd);
#ifdef NDEBUG
	try {
#endif
		WorkManager work_manager(instance->source_setting,
		                         instance->slipstream_setting,
		                         instance->output_setting);
		work_manager.addEventListener(*instance);
		work_manager.execute();

		instance->MessageBox(L"łI", L"F6c[", MB_OK | MB_ICONINFORMATION);
		// {^݂̂ɂ
		instance->PostMessage(UWM_ENABLE_FINISH_BUTTON, 0, 0);
#ifdef NDEBUG
	} catch (MyLib::FileTransferException<wchar_t>& e) {
		std::wostringstream ss;
		ss << L"From: " << e.getSourceFileName() << L"\n";
		ss << L"To: " << e.getDestinationFileName() << L"\n";
		if (e.getErrorFile() != NULL) {
			ss << e.getErrorFile() << L"(" << e.getErrorLine() << L")";
		}
		ss << e.getErrorText() << L'[' << e.getErrorCode() << L']';
		edit_ctrl << _T("\n") << ss.str() << std::endl;
		instance->MessageBox(ss.str().c_str(), NULL, MB_ICONERROR);
		instance->PostMessage(UWM_ENABLE_BACK_BUTTON, 0, 0);
	} catch (MyLib::FileException<wchar_t>& e) {
		std::wostringstream ss;
		ss << e.getFileName() << L"\n";
		if (e.getErrorFile() != NULL) {
			ss << e.getErrorFile() << L"(" << e.getErrorLine() << L")";
		}
		ss << e.getErrorText() << L'[' << e.getErrorCode() << L']';
		edit_ctrl << _T("\n") << ss.str() << std::endl;
		instance->MessageBox(ss.str().c_str(), NULL, MB_ICONERROR);
		instance->PostMessage(UWM_ENABLE_BACK_BUTTON, 0, 0);
	} catch (MyLib::Exception<wchar_t>& e) {
		std::wostringstream ss;
		if (e.getErrorFile() != NULL) {
			ss << e.getErrorFile() << L"(" << e.getErrorLine() << L")";
		}
		ss << e.getErrorText() << L'[' << e.getErrorCode() << L']';
		edit_ctrl << _T("\n") << ss.str() << std::endl;
		instance->MessageBox(ss.str().c_str(), NULL, MB_ICONERROR);
		instance->PostMessage(UWM_ENABLE_BACK_BUTTON, 0, 0);
	} catch (const std::runtime_error& e) {
		std::wstring err_msg = MyLib::convertUTF8ToUTF16(e.what());
		edit_ctrl << _T("\n") << err_msg << std::endl;
		instance->MessageBox(err_msg.c_str(), NULL, MB_ICONERROR);
		instance->PostMessage(UWM_ENABLE_BACK_BUTTON, 0, 0);
	}
#endif
	return 0;
}

BOOL ProgressPage::OnInitDialog(HWND /*wndFocus*/, LPARAM /*lInitParam*/)
{
#ifdef MY_DEBUG
	::WritePrivateProfileString(L"Debug", L"State", L"ProgressPage::OnInitDialog() start", PROFILE_PATH->string().c_str());
#endif
	this->edit1 = this->GetDlgItem(IDC_EDIT1);
	this->DlgResize_Init(false);
#ifdef MY_DEBUG
	::WritePrivateProfileString(L"Debug", L"State", L"ProgressPage::OnInitDialog() end", PROFILE_PATH->string().c_str());
#endif
	return true;
}

LRESULT ProgressPage::OnEnableFinishButton(unsigned int /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	return ::SendMessage(this->GetParent(), UWM_ENABLE_FINISH_BUTTON, 0, 0);
}

LRESULT ProgressPage::OnEnableBackButton(unsigned int /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	return ::SendMessage(this->GetParent(), UWM_ENABLE_BACK_BUTTON, 0, 0);
}
