#include "SourceSettingPage.h"
#include "core/SourceSetting.h"
#include "mylib/exception.h"
#include "mylib/wtlext/folderdialog.h"

#ifndef SHCNRF_InterruptLevel
#define SHCNRF_InterruptLevel      0x0001
#endif
#ifndef SHCNRF_ShellLevel
#define SHCNRF_ShellLevel          0x0002
#endif

BOOL SourceSettingPage::OnInitDialog(HWND /*wndFocus*/, LPARAM /*lInitParam*/)
{
#ifdef MY_DEBUG
	::WritePrivateProfileString(L"Debug", L"State", L"SourceSettingPage::OnInitDialog() start", PROFILE_PATH->string().c_str());
#endif

	// u}CRs[^vPIDL擾
	ITEMIDLIST *pidl;
	::SHGetSpecialFolderLocation(NULL, CSIDL_DRIVES, &pidl);
	// CD̔ʒm悤ɂ
	SHChangeNotifyEntry cne;
	cne.fRecursive = FALSE;
	cne.pidl = pidl;
	::SHChangeNotifyRegister(this->m_hWnd, SHCNRF_InterruptLevel | SHCNRF_ShellLevel,
		SHCNE_DRIVEADD | SHCNE_DRIVEREMOVED | SHCNE_MEDIAINSERTED | SHCNE_MEDIAREMOVED, UWM_CHANGE_NOTIFY, 1, &cne);
	::ILFree(pidl);

	this->cd_drive_list = this->GetDlgItem(IDC_LIST_DRIVE);
	this->enumerateCDDrives(/*-1*/);
	this->reload_button = this->GetDlgItem(IDC_BUTTON_RELOAD);
	this->folder_box = this->GetDlgItem(IDC_EDIT_FOLDER);

	// Radio Button
	this->CheckRadioButton(IDC_RADIO_CD, IDC_RADIO_FOLDER, IDC_RADIO_CD);
	this->switchControlsByRadioButton(IDC_RADIO_CD);

	this->DlgResize_Init(false);

#ifdef MY_DEBUG
	::WritePrivateProfileString(L"Debug", L"State", L"SourceSettingPage::OnInitDialog() end", PROFILE_PATH->string().c_str());
#endif
	return TRUE;
}

SourceSetting* SourceSettingPage::getSetting()
{
	SourceSetting *setting = NULL;

	this->DoDataExchange(DDX_SAVE);

	try {
		switch (this->radio1) {
		case 0:
			{
				const int drive_selection = this->cd_drive_list.GetCurSel();
				if (drive_selection == -1) {
					this->MessageBox(_T("CDhCuI"));
					return NULL;
				}
				wchar_t buf[] = L"A:/";
				// TODO: CDĂ邩`FbN
				buf[0] = static_cast<wchar_t>(this->cd_drive_list.GetItemData(drive_selection));
				setting = new DiscSourceSetting(buf);
			}
			break;
		case 1:
			{
				const int buf_count = this->folder_box.GetWindowTextLength() + 1;
				if (buf_count > 1) {
					boost::scoped_array<wchar_t> buf(new wchar_t[buf_count]);
					this->folder_box.GetWindowText(buf.get(), buf_count);
					boost::filesystem::wpath p(buf.get());
					setting = new DirectorySourceSetting(p);
				} else {
					this->MessageBox(_T("tH_w肭"));
					return NULL;
				}
			}
			break;
		//case 2:
		//	break;
		default:
			ATLASSERT(false);
			break;
		}
	} catch (MyLib::FileTransferException<wchar_t>& e) {
		std::wostringstream ss;
		ss << L"From: " << e.getSourceFileName() << L"\n";
		ss << L"To: " << e.getDestinationFileName() << L"\n";
		if (e.getErrorFile() != NULL) {
			ss << e.getErrorFile() << L"(" << e.getErrorLine() << L")";
		}
		ss << e.getErrorText();
		this->MessageBox(ss.str().c_str());
	} catch (MyLib::FileException<wchar_t>& e) {
		std::wostringstream ss;
		ss << e.getFileName() << L"\n";
		if (e.getErrorFile() != NULL) {
			ss << e.getErrorFile() << L"(" << e.getErrorLine() << L")";
		}
		ss << e.getErrorText();
		this->MessageBox(ss.str().c_str());
	} catch (MyLib::Exception<wchar_t>& e) {
		std::wostringstream ss;
		if (e.getErrorFile() != NULL) {
			ss << e.getErrorFile() << L"(" << e.getErrorLine() << L")";
		}
		ss << e.getErrorText() << L'[' << e.getErrorCode() << L']';
		this->MessageBox(ss.str().c_str());
	} catch (std::exception &e) {
		::MessageBoxA(this->m_hWnd, e.what(), NULL, 0);
	}

	//if (this->IsDlgButtonChecked(IDC_RADIO_CD) == BST_CHECKED) {
	//	const int drive_selection = this->cd_drive_list.GetCurSel();
	//	if (drive_selection == -1) {
	//		this->MessageBox(_T("CDhCuI"));
	//		return -1;
	//	}
	//	// Volume label & CD Check
	//	switch (setting.setCDSource(static_cast<TCHAR>(this->cd_drive_list.GetItemData(drive_selection)))) {
	//	case SourceSetting::SET_CD_SOURCE_NO_CD:
	//		this->MessageBox(_T("CDĂ܂"));
	//		return -1;
	//	case SourceSetting::SET_CD_SOURCE_INVALID:
	//		this->MessageBox(_T("CDɂ͑ΉĂ܂"));
	//		return -1;
	//	default:
	//		break;
	//	}
	//} else {
	//	const int buf_count = this->folder_box.GetWindowTextLength() + 1;
	//	if (buf_count > 0) {
	//		MyLib::TemporaryBuffer<TCHAR> buf(buf_count);
	//		this->folder_box.GetWindowText(buf.getPointer(), buf_count);
	//		MyLib::PathString path(buf.getPointer());
	//		if (!setting.setFolderSource(path)) {
	//			this->MessageBox(_T("̃tH_͎gpł܂"));
	//			return -1;
	//		}
	//	} else {
	//		this->MessageBox(_T("tH_w肭"));
	//		return -1;
	//	}
	//}
//
//	// eEBhE(vpeBV[g)ɒʒm
//	::SendMessage(this->GetParent(), UWM_SOURCE_SETTING_PAGE_NEXT, reinterpret_cast<WPARAM>(&setting), 0);
//	// u߂v{^\
//	::ShowWindow(::GetDlgItem(this->GetParent(), ID_WIZBACK), SW_SHOW);
//	// 0  = goto next page
//	// -1 = prevent page change
//	// >0 = jump to page by dlg ID
	return setting;
}

LRESULT SourceSettingPage::OnChangeNotify(unsigned int /*uMsg*/, WPARAM wParam, LPARAM lParam)
{
	ITEMIDLIST **pidl_array = reinterpret_cast<ITEMIDLIST**>(wParam);
	ITEMIDLIST *pidl = pidl_array[0];
	TCHAR buf[MAX_PATH] = {0};
	if (::SHGetPathFromIDList(pidl, buf)) {
		switch (lParam) {
		case SHCNE_DRIVEADD:
		case SHCNE_MEDIAINSERTED:
			if (::GetDriveType(buf) == DRIVE_CDROM) {
				this->enumerateCDDrives();
			}
			break;
		// hCu폜ŁASHCNE_DRIVEREMOVEDł͂ȂASHCNE_MEDIAREMOVED?
		case SHCNE_MEDIAREMOVED:
		case SHCNE_DRIVEREMOVED:
			for (int i = 0; i < this->cd_drive_list.GetCount(); ++i) {
				if (this->cd_drive_list.GetItemData(i) == buf[0]) {
					this->enumerateCDDrives();
					break;
				}
			}
			break;
		default:
			break;
		}
	}
	return 0;
}

// R{{bNXCDhCu񋓂
void SourceSettingPage::enumerateCDDrives(/*int prev_selection*/)
{
	// ݁AIĂ鍀ڂL
	int const drive_selection = this->cd_drive_list.GetCurSel();
	TCHAR const selected_drive_letter =
		(drive_selection != CB_ERR ? static_cast<TCHAR>(this->cd_drive_list.GetItemData(drive_selection)) : 0);
	// ڂS
	this->cd_drive_list.ResetContent();
	// ڐݒpf[^
    struct CDDriveData {
        TCHAR drive_letter;
        HICON icon;
        std::wstring display_name;
    };
    std::vector<CDDriveData> cd_drives;
	// ݂ĂSCD/DVDhCu擾
    //size_t required_count = ::GetLogicalDriveStrings(0, NULL);
    //MyLib::TemporaryBuffer<TCHAR> temp_buf(required_count);
    //::GetLogicalDriveStrings(required_count, temp_buf.getPointer());
    //TCHAR *ptr = temp_buf.getPointer();
    //size_t n1; 
    //while ((n1 = ::_tcslen(ptr)) > 0) {
    DWORD const drive_mask = ::GetLogicalDrives();
    for (char i = 0; i < 26; ++i) {
		if (drive_mask & (1 << i)) {
			const TCHAR drive_letter = 'A' + i;
			const TCHAR root_path[] = {drive_letter, ':', '\\', '\0'};
			if (::GetDriveType(/*ptr*/root_path) == DRIVE_CDROM) {
				CDDriveData drive_data;
				drive_data.drive_letter = /**ptr;*/drive_letter;
				SHFILEINFO file_info;
				::SHGetFileInfo(/*ptr*/root_path, 0, &file_info, sizeof(SHFILEINFO), SHGFI_DISPLAYNAME | SHGFI_ICON | SHGFI_SMALLICON);
				drive_data.icon = file_info.hIcon;
				drive_data.display_name = file_info.szDisplayName;
				cd_drives.push_back(drive_data);
			}
		}
        //ptr += n1 + 1;
    }
	// hCũACRC[WXgɒǉ
	CImageList image_list;
	image_list.Create(16, 16, ILC_COLOR16 | ILC_MASK, cd_drives.size(), 0);
	image_list.SetBkColor(RGB(255, 255, 255));
	for (size_t i = 0; i < cd_drives.size(); ++i) {
		image_list.AddIcon(cd_drives[i].icon);
	}
	this->cd_drive_list.SetImageList(image_list);
	// hCuꗗɍڂǉ
	int new_drive_selection = 0;
	for (size_t i = 0; i < cd_drives.size(); ++i) {
		COMBOBOXEXITEM cbei;
		cbei.mask = CBEIF_TEXT | CBEIF_IMAGE | CBEIF_SELECTEDIMAGE;
		cbei.pszText = const_cast<wchar_t*>(cd_drives[i].display_name.c_str());
		cbei.cchTextMax = cd_drives[i].display_name.length();
		cbei.iItem = i;
		cbei.iImage = i;
		cbei.iSelectedImage = i;
		int current_item_index = this->cd_drive_list.InsertItem(&cbei);
		// ڂ̕tf[^ɃhCu
		this->cd_drive_list.SetItemData(current_item_index, cd_drives[i].drive_letter);
		// đIĂhCuƈv鍀
		if (cd_drives[i].drive_letter == selected_drive_letter) {
			new_drive_selection = current_item_index;
		}
	}
	// đIĂhCuĂёI悤ɂ
	const int cd_drive_count = this->cd_drive_list.GetCount();
	if (cd_drive_count > 0) {
		this->cd_drive_list.SetCurSel(new_drive_selection);
	}
}

void SourceSettingPage::OnRadioCd(unsigned int /*uNotifyCode*/, int nID, HWND /*wndCtl*/)
{
	this->switchControlsByRadioButton(/*IDC_RADIO_CD*/nID);
}

void SourceSettingPage::OnRadioFolder(unsigned int /*uNotifyCode*/, int nID, HWND /*wndCtl*/)
{
	this->switchControlsByRadioButton(/*IDC_RADIO_FOLDER*/nID);
}

void SourceSettingPage::OnRadioPreinstall(unsigned int /*uNotifyCode*/, int nID, HWND /*wndCtl*/)
{
	this->switchControlsByRadioButton(nID);
}

// WI{^̑IɂāAeRg[̐؂ւs
void SourceSettingPage::switchControlsByRadioButton(int radio_button_id)
{
	this->cd_drive_list.EnableWindow(radio_button_id == IDC_RADIO_CD);
	this->folder_box.EnableWindow(radio_button_id == IDC_RADIO_FOLDER);
	::EnableWindow(this->GetDlgItem(IDC_BUTTON_FOLDER),
		radio_button_id == IDC_RADIO_FOLDER);
}

void SourceSettingPage::OnButtonFolder(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	// tH_̎Q
	static std::wstring initial_dir_path;
	MyLib::FolderDialog dlg(initial_dir_path, this->m_hWnd, NULL,
		BIF_RETURNONLYFSDIRS | BIF_NEWDIALOGSTYLE | BIF_NONEWFOLDERBUTTON);
	if (dlg.DoModal() == IDOK) {
		this->folder_box.SetWindowText(dlg.GetFolderPath());
		initial_dir_path = dlg.GetFolderPath();
	}
}

void SourceSettingPage::OnButtonReload(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	this->enumerateCDDrives();
}
