﻿#include "WorkDirSettingPage.h"
#include "core/SourceSetting.h"
#include "mylib/function.h"
#include "mylib/wtlext/folderdialog.h"
#include "mylib/wtlext/textextent.h"

WorkDirSettingPage::WorkDirSettingPage()
: is_source_read_only(false)
, no_work_dir(false)
, temp_work_dir(false)
{
	//this->dir_path[0] = L'\0';
}

//boost::filesystem::wpath* WorkDirSettingPage::getSetting()
WorkDirSetting* WorkDirSettingPage::getSetting()
{
	this->DoDataExchange(DDX_SAVE);

	namespace fs = boost::filesystem;

	if (this->no_work_dir) {
		WorkDirSetting *result = new WorkDirSetting();
		result->creates_temp_dir = false;
		return result;
	}

	// Work Dir
	fs::wpath p = this->dir_path;

	// 
	if (this->temp_work_dir) {
		std::wstring str = p.string();
		// フォルダのパスが空欄
		if (str.empty()) {
			uint64_t max_free_space = 0;
			const uint32_t drive_mask = ::GetLogicalDrives();
			for (char i = 0; i < 26; ++i) {
				if (drive_mask & (1 << i)) {
					wchar_t const drive_letter = L'A' + i;
					wchar_t const root_path[] = {drive_letter, L':', L'\\', L'\0'};
					if (::GetDriveType(root_path) == DRIVE_FIXED) {
						// 空き容量の取得
						ULARGE_INTEGER uli;
						if (::GetDiskFreeSpaceEx(root_path, &uli, NULL, NULL)
						 && uli.QuadPart > max_free_space) {
							str = root_path;
							max_free_space = uli.QuadPart;
						}
					}
				}
			}
		}
		WorkDirSetting *result = new WorkDirSetting();
		result->work_dir_path = str;
		result->creates_temp_dir = true;
		return result;
		//return new fs::wpath(MyLib::createTemporaryFileName(str.c_str()));
	}

	if (p.string().empty()) {
		this->MessageBox(_T("作業フォルダをご指定ください"));
		return NULL;
	}

	if (fs::exists(p)) {
		// 存在していれば、ディレクトリか、中が空かチェック
		if (fs::is_directory(p)) {
			// 空かどうか
			if (!fs::is_empty(p)) {
				int ret = this->MessageBox(
					_T("作業フォルダは空ではありませんが、よろしいですか？"),
					_T("確認"), MB_OKCANCEL | MB_ICONQUESTION | MB_DEFBUTTON2);
				if (ret != IDOK) {
					return NULL;
				}
			}
		} else {
			// ディレクトリじゃない
			this->MessageBox(_T("ご指定のパスはフォルダではありません"), _T("作業フォルダ設定中のエラー"));
			return NULL;
		}
	}
	WorkDirSetting *result = new WorkDirSetting();
	result->work_dir_path = p;
	result->creates_temp_dir = false;
	return result;
}

void WorkDirSettingPage::loadSourceSetting(SourceSetting const *source_setting)
{
	this->is_source_read_only = source_setting->isReadOnly();
	if (this->is_source_read_only) {
		this->no_work_dir = false;
		this->DoDataExchange(DDX_LOAD);
		this->no_work_dir_checkbox.EnableWindow(false);
	} else if (!this->temp_work_dir) {
		this->no_work_dir_checkbox.EnableWindow(true);
	}
}

/////////////////////////////////////////////////////////////////////////////

BOOL WorkDirSettingPage::PreTranslateMessage(MSG * /*pMsg*/)
{
	return FALSE;
}

BOOL WorkDirSettingPage::OnInitDialog(HWND /*wndFocus*/, LPARAM /*lInitParam*/)
{
	static wchar_t const *COL0_TEXT = L"ドライブ";
	static wchar_t const *COL1_TEXT = L"空き容量";

#ifdef MY_DEBUG
	::WritePrivateProfileString(L"Debug", L"State", L"WorkDirSettingPage::OnInitDialog() start", PROFILE_PATH->string().c_str());
#endif

	this->folder_box = this->GetDlgItem(IDC_EDIT_FOLDER);
	this->folder_button = this->GetDlgItem(IDC_BUTTON_FOLDER);
	this->no_work_dir_checkbox = this->GetDlgItem(IDC_CHECKBOX_NO_WORK_DIR);
	this->temp_work_dir_checkbox = this->GetDlgItem(IDC_CHECKBOX_TEMP_WORK_DIR);
	this->free_space_list = this->GetDlgItem(IDC_LIST_FREE_SPACE);
	this->free_space_list.InsertColumn(0, COL0_TEXT, LVCFMT_LEFT);
	this->free_space_list.InsertColumn(1, COL1_TEXT, LVCFMT_RIGHT);

	SHFILEINFO sh_file_info;
	HIMAGELIST hil = reinterpret_cast<HIMAGELIST>(::SHGetFileInfo(
		L"", 0, &sh_file_info, sizeof(SHFILEINFO), SHGFI_SYSICONINDEX | SHGFI_SMALLICON));
	this->free_space_list.SetImageList(hil, LVSIL_SMALL);

	//// リストビューのDCとフォントの取得
	//CDC list_view_dc = this->free_space_list.GetDC();
	//HFONT prev_font = list_view_dc.SelectFont(this->free_space_list.GetFont());
	// 列の名前を表示するのに必要な幅を取得する
	TextExtent text_extent(this->free_space_list);
	//SIZE text_extent;
	//list_view_dc.GetTextExtent(COL1_TEXT, std::wcslen(COL0_TEXT), &text_extent);
	//int col0_max_width = text_extent.cx;
	//list_view_dc.GetTextExtent(COL1_TEXT, std::wcslen(COL1_TEXT), &text_extent);
	//int col1_max_width = text_extent.cx;
	int col0_max_width = text_extent.getTextExtent(COL0_TEXT).cx;
	int col1_max_width = text_extent.getTextExtent(COL1_TEXT).cx;

	std::wstringstream ss;
	// Localeを設定することで、数値の桁区切りがされる
	ss.imbue(std::locale(""));

	const uint32_t drive_mask = ::GetLogicalDrives();
	for (char i = 0; i < 26; ++i) {
		if (drive_mask & (1 << i)) {
			const TCHAR drive_letter = 'A' + i;
			const TCHAR root_path[] = {drive_letter, ':', '\\', '\0'};
			if (::GetDriveType(root_path) != DRIVE_FIXED) {
				continue;
			}
			// 空き容量の取得
			ULARGE_INTEGER uli;
			if (!::GetDiskFreeSpaceEx(root_path, &uli, NULL, NULL)) {
				continue;
			}
			// アイコンと表示名の取得
			::SHGetFileInfo(root_path, 0, &sh_file_info, sizeof(SHFILEINFO), SHGFI_DISPLAYNAME | SHGFI_SYSICONINDEX | SHGFI_SMALLICON);
			const int item_count = this->free_space_list.GetItemCount();
			this->free_space_list.InsertItem(item_count, sh_file_info.szDisplayName, sh_file_info.iIcon);

			ss.str(_T(""));
			ss << uli.QuadPart/(1024*1024) << L" MB";
			std::wstring const str = ss.str();
			this->free_space_list.SetItemText(item_count, 1, str.c_str());
			// 表示に必要な幅を取得する
			//SIZE text_extent;
			//list_view_dc.GetTextExtent(sh_file_info.szDisplayName, ::_tcslen(sh_file_info.szDisplayName), &text_extent);
			//if (col0_max_width < text_extent.cx) {
			//	col0_max_width = text_extent.cx;
			//}
			//list_view_dc.GetTextExtent(str.c_str(), str.length(), &text_extent);
			//if (col1_max_width < text_extent.cx) {
			//	col1_max_width = text_extent.cx;
			//}
			int const col0_width = text_extent.getTextExtent(sh_file_info.szDisplayName).cx;
			if (col0_max_width < col0_width) {
				col0_max_width = col0_width;
			}
			int const col1_width = text_extent.getTextExtent(str).cx;
			if (col1_max_width < col1_width) {
				col1_max_width = col1_width;
			}
		}
	}

	col0_max_width += 32;
	col1_max_width += 16;
	//list_view_dc.SelectFont(prev_font);
	// 全項目を追加してからクライアント領域を求めているので、
	// 垂直スクロールバーがあっても大丈夫
	RECT list_view_rect;
	this->free_space_list.GetClientRect(&list_view_rect);
	const int list_view_width = list_view_rect.right - list_view_rect.left;
	// 空き容量が完全に表示されるように列幅を設定
	this->free_space_list.SetColumnWidth(1, col1_max_width);
	const int col0_width =
		(col0_max_width + col1_max_width <= list_view_width
		? col0_max_width : list_view_width - col1_max_width);
	this->free_space_list.SetColumnWidth(0, col0_width);

	this->DlgResize_Init(false);

	this->DoDataExchange(DDX_LOAD);

#ifdef MY_DEBUG
	::WritePrivateProfileString(L"Debug", L"State", L"WorkDirSettingPage::OnInitDialog() end", PROFILE_PATH->string().c_str());
#endif
	return TRUE;
}

void WorkDirSettingPage::OnButtonFolder(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	static std::wstring initial_dir_path;

	MyLib::FolderDialog dlg(initial_dir_path, this->m_hWnd);//, ptr);
	if (dlg.DoModal() == IDOK) {
		this->folder_box.SetWindowText(dlg.GetFolderPath());
		initial_dir_path = dlg.GetFolderPath();
	}
}

void WorkDirSettingPage::OnCheckboxNoWorkDir(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	this->DoDataExchange(DDX_SAVE);

	ATLASSERT(!this->is_source_read_only && !this->temp_work_dir);

	if (this->no_work_dir) {
		this->folder_box.EnableWindow(false);
		this->folder_button.EnableWindow(false);
		//this->temp_work_dir = false;
		//this->DoDataExchange(DDX_LOAD);
		this->temp_work_dir_checkbox.EnableWindow(false);
	} else {
		this->folder_box.EnableWindow(true);
		this->folder_button.EnableWindow(true);
		this->temp_work_dir_checkbox.EnableWindow(true);
	}
}

void WorkDirSettingPage::OnCheckboxTempWorkDir(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	this->DoDataExchange(DDX_SAVE);

	ATLASSERT(!this->no_work_dir);

	if (this->temp_work_dir) {
		//this->no_work_dir = false;
		//this->DoDataExchange(DDX_LOAD);
		this->no_work_dir_checkbox.EnableWindow(false);
	} else if (!this->is_source_read_only) {
		this->no_work_dir_checkbox.EnableWindow(true);
	}
}
