#include "DriverFileData.h"
#include "DriverSetupData.h"
#include "mylib/FileCompressionInfo.h"

namespace fs {
	using namespace boost::filesystem;
};

namespace {
fs::wpath getRelativePath(fs::wpath const &p1, fs::wpath const &p2)
{
	size_t p2_len = p2.string().length();
	fs::wpath tmp = p1;
	std::list<std::wstring> names;
	for (;;) {
		if (tmp.string().length() < p2_len) {
			return L"";
		}
		names.push_front(tmp.filename());
		tmp = tmp.parent_path();
		if (fs::equivalent(tmp, p2)) {
			break;
		}
	}
	fs::wpath result;
	std::list<std::wstring>::const_iterator it = names.begin();
	for (; it != names.end(); ++it) {
		result /= *it;
	}
	return result;
}
};

void DriverFileData::addFiles(DriverSetupData const &data, fs::wpath const &sub_dir_path)
{
	std::list<fs::wpath> files;
	fs::wpath const inf_path = data.getInfFilePath();
	fs::wpath const dir_path = inf_path.parent_path();
	// 폜Ȃt@CXgINFt@Cǉ
	files.push_back(inf_path);
	// CINFt@CCopyInfsŎw肳Ăt@C̏
	DriverSetupData::CopyInfs::const_iterator it37 = data.getCopyInfs().begin();
	for (; it37 != data.getCopyInfs().end(); ++it37) {
		// INFt@C͂̂܂܃Rs[
		files.push_back(dir_path/(it37->second));
	}
	// NewCabFile ǉ
	DriverSetupData::NewCabFileCollection const new_cab_files = data.getNewCabFiles();
	DriverSetupData::NewCabFileCollection::const_iterator it36 = new_cab_files.begin();
	for (; it36 != new_cab_files.end(); ++it36) {
		files.push_back(dir_path/(it36->first));
	}
	// TagFilẽRs[
	DriverSetupData::TagFileCollection const tag_files = data.getTagFiles();
	DriverSetupData::TagFileCollection::const_iterator it35 = tag_files.begin();
	for (; it35 != tag_files.end(); ++it35) {
		files.push_back(dir_path/(it35->second));
	}
	// CINFt@CCopyFilesŎw肳Ăt@C̏
	DriverSetupData::FileDataCollection::const_iterator it3 =
		data.getFileDataCollection().begin();
	for (; it3 != data.getFileDataCollection().end(); ++it3) {
		DriverSetupData::CopyFilesTagData::const_iterator it2 =
			it3->second.begin();
		for (; it2 != it3->second.end(); ++it2) {
			std::wstring const file_title = it2->first;
			fs::wpath const source_file_path = it2->second.source_path;
			MyLib::FileCompressionInfo info(source_file_path);
			files.push_back(info.getActualPath());
		}
	}
	// catt@C
	fs::wpath const cat_file_path = data.getCatalogFilePath();
	if (fs::exists(cat_file_path)) {
		files.push_back(cat_file_path);
	}

	std::list<fs::wpath>::const_iterator it1 = files.begin();
	for (; it1 != files.end(); ++it1) {
		bool exists = false;
		std::list<fs::wpath>::const_iterator it2 = this->files_.begin();
		for (; it2 != this->files_.end(); ++it2) {
			if (fs::equivalent(*it1, sub_dir_path/(*it2))) {
				exists = true;
				break;
			}
		}
		if (!exists) {
			fs::wpath const rel_path = getRelativePath(*it1, sub_dir_path);
			assert(!rel_path.empty());
			this->files_.push_back(rel_path);
		}
	}
}

std::list<boost::filesystem::wpath> const& DriverFileData::getFileList() const
{
	return this->files_;
}
