#include "FileCopyManager.h"
#include "mylib/function.h"

namespace fs {
using namespace boost::filesystem;
};

FileCopyManager::FileCopyManager(boost::filesystem::wpath const &src_dir_path,
                                 boost::filesystem::wpath const &dst_dir_path)
	: src_dir_path(src_dir_path)
	, dst_dir_path(dst_dir_path)
{
}

// t@CRs[(葤Ǝ󂯑ňقȂt@C)
void FileCopyManager::copyFile(std::wstring const &src_filename, std::wstring const &dst_filename) const
{
	fs::wpath const src_path = this->src_dir_path/src_filename;
	fs::wpath const dst_path = this->dst_dir_path/dst_filename;
	std::wstring const src_path_str = src_path.string();
	std::wstring const dst_path_str = dst_path.string();
	MyLib::toFileWritable(dst_path);
	if (!::CopyFile(src_path_str.c_str(), dst_path_str.c_str(), FALSE)) {
		throw MY_WIN32_FILE_TRANSFER_EXCEPTION_LAST_ERROR(src_path_str, dst_path_str);
	}
}

// t@CRs[(葤Ǝ󂯑œt@C)
void FileCopyManager::copyFile(std::wstring const &filename) const
{
	this->copyFile(filename, filename);
}

// t@CRs[(ChJ[hgp\)
void FileCopyManager::copyFilesUsingWildCards(std::wstring const &filename) const
{
	fs::wpath const wild_card = this->src_dir_path/filename;
	WIN32_FIND_DATA fd;
	HANDLE h = ::FindFirstFile(wild_card.string().c_str(), &fd);
	if (h != INVALID_HANDLE_VALUE) {
		do {
			if (!(fd.cFileName[0] == _T('.') && (fd.cFileName[1] == _T('\0') || (fd.cFileName[1] == _T('.') && fd.cFileName[2] == _T('\0'))))) {
				fs::wpath const src_file_path = this->src_dir_path/fd.cFileName;
				fs::wpath const dst_file_path = this->dst_dir_path/fd.cFileName;
				MyLib::toFileWritable(dst_file_path);
				if (!::CopyFile(src_file_path.string().c_str(), dst_file_path.string().c_str(), FALSE)) {
					::FindClose(h);
					throw MY_WIN32_FILE_TRANSFER_EXCEPTION_LAST_ERROR(src_file_path.string(), dst_file_path.string());
				}
			}
		} while (::FindNextFile(h, &fd));
		::FindClose(h);
	}
}


void FileCopyManager::changeSourceDirectory(boost::filesystem::wpath const &dir_path)
{
	this->src_dir_path = dir_path;
}

void FileCopyManager::changeDestinationDirectory(boost::filesystem::wpath const &dir_path)
{
	this->dst_dir_path = dir_path;
}
