#pragma once

//===========================================================================
// ModelsTargetOSVer
//	INFt@CModelsZNVOSo[W
//===========================================================================
class ModelsTargetOSVer
{
	std::wstring os_ver_str;

	enum ArchitectureFlag {
		X86 = 1,
		AMD64 = 2,
		IA64 = 4,
	};

public:
	ModelsTargetOSVer() {}

	ModelsTargetOSVer(const std::wstring& str)
		: os_ver_str(str)
	{}

	std::wstring getVerString() const
	{
		return this->os_ver_str.empty() ? _T("") : (_T('.') + this->os_ver_str);
	}

	unsigned int getArchitectureFlag() const
	{
		// '.'̈ʒu܂ł̕擾
		const std::wstring architecture_str = tokenize(this->os_ver_str, _T('.'), 0);
		// x64phCo NTamd64 w肳ĂƂ̂݃CXg[̂AMD64(x64)͏O
		unsigned int architecture_flag = static_cast<unsigned int>(~AMD64);
		using MyLib::compareStringNoCase;
		if (compareStringNoCase(architecture_str, _T("nt")) == 0) {
			architecture_flag = X86 /*| AMD64*/ | IA64;
		} else if (compareStringNoCase(architecture_str, _T("ntx86")) == 0) {
			architecture_flag = X86;
		} else if (compareStringNoCase(architecture_str, _T("ntamd64")) == 0) {
			architecture_flag = AMD64;
		} else if (compareStringNoCase(architecture_str, _T("ntia64")) == 0) {
			architecture_flag = IA64;
		}
		return architecture_flag;
	}

	uint32_t getVersion() const
	{
		uint16_t major_ver = 0;
		uint16_t minor_ver = 0;
		std::wstring str = tokenize(this->os_ver_str, _T('.'), 1);
		if (!str.empty()) {
			major_ver = static_cast<uint16_t>(::_tstoi(str.c_str()));
		}
		str = tokenize(this->os_ver_str, _T('.'), 2);
		if (!str.empty()) {
			minor_ver = static_cast<uint16_t>(::_tstoi(str.c_str()));
		}
		return MAKELONG(minor_ver, major_ver);
	}

	// ؂蕶ŕAw肳ꂽԂ̕Ԃ
	static std::wstring tokenize(
		const std::wstring& str,
		TCHAR token,
		unsigned int index)
	{
		using std::wstring;
		std::wstring::size_type target_pos = 0;
		std::wstring::size_type period_pos = str.find(token, target_pos);
		if (index > 0) {
			for (unsigned int i = 0; i < index; ++i) {
				if (period_pos == std::wstring::npos) {
					return _T("");
				}
				target_pos = period_pos + 1;
				period_pos = str.find(token, target_pos);
			}
		}
		std::wstring::size_type length =
			(period_pos != std::wstring::npos ? period_pos - target_pos : std::wstring::npos);
		return str.substr(target_pos, length);
	}

	static unsigned int getArchitectureFlagFromString(const std::wstring& str)
	{
		if (str == _T("X86")) {
			return X86;
		} else if (str == _T("AMD64")) {
			return AMD64;
		} else if (str == _T("IA64")) {
			return IA64;
		}
		return 0;
	}
};

//===========================================================================
// InfFileParser
//	INFt@ChCoCXg[擾
//===========================================================================
class InfFileParser
{
public:
	// RXgN^
	InfFileParser(
		const std::wstring& filename,
		int target_architecture,
		uint32_t target_version);
	// INFt@C̃tpX擾
	std::wstring getFullPathName();
	// INFt@C̐
	std::wstring getProviderName();
	std::wstring getClassGUID();
	std::wstring getDateString();
	std::wstring getVersionString();
	// INFt@CCatalogt@C擾
	std::wstring getCatalogFileTitle();

	// Models
	typedef std::map<std::wstring, ModelsTargetOSVer> ModelsCollection;
	ModelsCollection getModelsSections();

	DriverSetupData::DDInstallCollection
		getDDInstallSections(const std::wstring& models_section);
	//
	DriverSetupData::ServiceDataCollection::value_type
		getServiceData(const std::wstring& dd_install_section);
	// INFt@CAhCõt@CRs[f[^擾B
	DriverSetupData::CopyFilesTags
		getCopyFilesTags(const std::wstring& dd_install_section);
	DriverSetupData::CopyFilesTagData
		getCopyFilesData(const std::wstring& copy_files_tag, bool is_installed_file = false);
	// INFt@CAINFt@C̃Rs[f[^擾B
	DriverSetupData::CopyInfs
		getCopyInfs(const std::wstring& dd_install_section, bool is_installed_file = false);
	DriverSetupData::TagFileCollection getTagFiles() const;
	DriverSetupData::NewCabFileCollection getNewCabFiles() const;

	// w肳ꂽZNVɁA.x86/.amd64/.ia64tꂽ̂ȂTB
	MyLib::InfContextIterator findDecoratedSection(LPCTSTR section, LPCTSTR key = NULL);
	// w肳ꂽZNVɁA.NT/.NTx86/.NTamd64/.NTia64
	// tꂽ̂ȂTB
	std::wstring findDecoratedSectionNameWithNT(const std::wstring& section);

	// VXeINFtH_ɂ"oem0.inf"Ȃǂ̌̃t@C߂
	std::wstring getOriginalFileTitleFromOEMInf();
	//static std::wstring getOriginalFileTitleFromOEMInf(const std::wstring& oem_inf_file_path);

private:
	// INFt@CIuWFNg
	MyLib::InfFile inf_file;
	// CPUA[LeN`(x86/AMD64/IA64)
	std::wstring architecture_str;
	// OS̃o[W
	uint32_t version;

	//
	bool matchesTargetMoreClosely(
		unsigned int new_architecture_flag, uint32_t new_version,
		unsigned int current_architecture_flag, uint32_t current_version);
	//
	DriverSetupData::FileData *getFileData(const std::wstring& filename);
	//
	DriverSetupData::FileData *getInstalledFileData(
		std::wstring const &inst_file_title, boost::filesystem::wpath const &src_file_path);
	//
	DriverSetupData::RegDataCollection getAddregData(
		const std::wstring& dd_install_service,
		const std::wstring& service_name);

	DriverSetupData::TagFileCollection tag_files;
	DriverSetupData::NewCabFileCollection new_cab_files;
};
