#include "SourceSizeReducer.h"
#include "DriverSetupData.h"
#include "TemporaryDirectory.h"
#include "mylib/OtherProcessExecutor.h"
#include "mylib/ConsoleProcessExecutor.h"

extern void deleteDirectory(boost::filesystem::wpath const &dir_path);

namespace {
	/**
	 * t@CEfBNg̈ړ(ChJ[hgp)
	 */
	void moveFilesUsingWildCards(boost::filesystem::wpath const &src_path,
	                             boost::filesystem::wpath const &dest_path,
	                             std::wstring const &file_name)
	{
		namespace fs = boost::filesystem;
		fs::wpath wild_card = src_path/file_name;
		WIN32_FIND_DATA fd;
		HANDLE h = ::FindFirstFile(wild_card.string().c_str(), &fd);
		if (h != INVALID_HANDLE_VALUE) {
			do {
				if (!(fd.cFileName[0] == _T('.') && (fd.cFileName[1] == _T('\0') || (fd.cFileName[1] == _T('.') && fd.cFileName[2] == _T('\0'))))) {
					fs::wpath const src_file_path = src_path/fd.cFileName;
					fs::wpath const dest_file_path = dest_path/fd.cFileName;
					if (!::MoveFile(src_file_path.string().c_str(), dest_file_path.string().c_str())) {
						throw MY_WIN32_FILE_TRANSFER_EXCEPTION_LAST_ERROR(src_file_path.string(), dest_file_path.string());
					}
				}
			} while (::FindNextFile(h, &fd));
			::FindClose(h);
		}
	}
	/**
	 * t@CEfBNg̈ړ(󂯑̃fBNgȂ΍쐬)
	 */
	void moveFilesWithDirectoryCreation(boost::filesystem::wpath const &src_path,
	                                    boost::filesystem::wpath const &dest_path)
	{
		namespace fs = boost::filesystem;
		fs::wpath dest_dir_path = dest_path;
		dest_dir_path.remove_filename();
		if (!fs::exists(dest_dir_path)) {
			fs::create_directories(dest_dir_path);
		}
		std::wstring const src_path_str = src_path.string();
		std::wstring const dest_path_str = dest_path.string();
		if (!::MoveFile(src_path_str.c_str(), dest_path_str.c_str())) {
			throw MY_WIN32_FILE_TRANSFER_EXCEPTION_LAST_ERROR(src_path_str, dest_path_str);
		}
	}
};

SourceSizeReducer::SourceSizeReducer(boost::filesystem::wpath const &work_dir_path,
                                     std::wstring const &setup_subdir_name,
                                     boost::filesystem::wpath const &temp_dir,
                                     uint32_t os_ver)
	: work_dir_path_(work_dir_path)
	, setup_subdir_name_(setup_subdir_name)
	, target_path_(work_dir_path/setup_subdir_name)
	, temp_dir_(temp_dir)
	, os_ver_(os_ver)
{
}

void SourceSizeReducer::execute()
{
	namespace fs = boost::filesystem;

	this->fireEvent(L"N[CXg[ɕKv̂Ȃt@C폜Ă܂BBB");

	// ꎞtH_쐬
	TemporaryDirectoryCloseUndoMovedFiles temp_dir_close_undo(this->work_dir_path_);
	TemporaryDirectoryCloseDelete temp_dir_close_delete;
	TemporaryDirectory temp_dir(this->temp_dir_, &temp_dir_close_undo);
	fs::wpath const temp_dir_path = temp_dir.getDirectoryPath();

	// St@CꎞtH_Ɉړ
	moveFilesUsingWildCards(this->work_dir_path_, temp_dir_path, L"*.*");

	//
	// DOSNET.INFɂĂȂt@CEfBNg߂
	//

	// SYSTEM32
	fs::create_directories(this->work_dir_path_/this->setup_subdir_name_);
	moveFilesUsingWildCards(temp_dir_path/this->setup_subdir_name_,
	                        this->work_dir_path_/this->setup_subdir_name_,
	                        L"SYSTEM32");
	// BOOTFIX.BIN
	moveFilesUsingWildCards(temp_dir_path/this->setup_subdir_name_,
	                        this->work_dir_path_/this->setup_subdir_name_,
	                        L"BOOTFIX.BIN");
	// WINNT.SIF
	moveFilesUsingWildCards(temp_dir_path/this->setup_subdir_name_,
	                        this->work_dir_path_/this->setup_subdir_name_,
	                        L"WINNT.SIF");
	// $OEM$
	moveFilesUsingWildCards(temp_dir_path,
	                        this->work_dir_path_,
	                        L"$OEM$");
	// BOOTFONT.BIN
	moveFilesUsingWildCards(temp_dir_path,
	                        this->work_dir_path_,
	                        L"BOOTFONT.BIN");
	// WIN51*
	moveFilesUsingWildCards(temp_dir_path,
	                        this->work_dir_path_,
	                        L"WIN51*");
	// CDROM*.*
	moveFilesUsingWildCards(temp_dir_path,
	                        this->work_dir_path_,
	                        L"CDROM*.*");

	// DOSNET.INFJ
	fs::wpath const dosnet_inf_path = temp_dir_path/this->setup_subdir_name_/L"dosnet.inf";
	this->dosnet_inf.reset(new MyLib::SetupFileReader(dosnet_inf_path.string()));

	// DOSNET.INF[Directories]ZNVAfBNgꗗ쐬
	MyLib::SetupFileLineIterator const it_end;
	MyLib::SetupFileLineIterator it = this->dosnet_inf->findFirstLine(L"Directories");
	if (this->os_ver_ == 0x00050000) {
		for (; it != it_end; ++it) {
			std::wstring const dir_name = L"\\" + this->setup_subdir_name_ + it->getStringField(1);
			directories[it->getStringField(0)] = dir_name;
		}
	} else {
		for (; it != it_end; ++it) {
			std::wstring const dir_name = it->getStringField(1);
			// cmpnentsfBNgO
			if (MyLib::compareStringNoCase(dir_name.substr(0, 10), L"\\cmpnents\\") != 0) {
				directories[it->getStringField(0)] = dir_name;
			}
		}
	}

	// DOSNET.INF[Files]ZNVɗ񋓂Ăt@C߂
	this->moveDosnetFiles(L"Files", temp_dir_path/*, directories*/);
	// DOSNET.INF[FloppyFiles.*]ZNVɗ񋓂Ăt@C߂
	int i = 0;
	std::wostringstream ss;
	do {
		ss.str(_T(""));
		ss << _T("FloppyFiles.") << i;
		++i;
	} while (this->moveDosnetFiles(ss.str(), temp_dir_path/*, directories*/));
	// DOSNET.INF[OptionalSrcDirs]ZNVɗ񋓂ĂfBNg߂
	this->moveDosnetDirectories(_T("OptionalSrcDirs"), temp_dir_path);
	// DOSNET.INF[AssemblyDirectories]ZNVɗ񋓂ĂfBNg߂
	this->moveDosnetDirectories(_T("AssemblyDirectories"), temp_dir_path);

	// DOSNET.INF
	this->dosnet_inf.reset(NULL);
	// DOSNET.INF߂
	moveFilesUsingWildCards(temp_dir_path/this->setup_subdir_name_, this->work_dir_path_/this->setup_subdir_name_, L"DOSNET.INF");

	// ꎞfBNg̍폜
	temp_dir.setCloseBehavior(&temp_dir_close_delete);

	this->fireEvent(L"\n");
}

bool SourceSizeReducer::moveDosnetFiles(std::wstring const &section_name,
                                        boost::filesystem::wpath const &temp_dir_path)
{
	namespace fs = boost::filesystem;
	typedef std::map<std::wstring, std::wstring, MyLib::LessNoCase> MapStringToString;

	MyLib::SetupFileLineIterator const it_end;
	MyLib::SetupFileLineIterator it = this->dosnet_inf->findFirstLine(section_name.c_str());
	if (it == it_end) {
		return false;
	}
	for (; it != it_end; ++it) {
		MapStringToString::const_iterator it2 =
			directories.find(it->getStringField(1));
		if (it2 == directories.end()) {
			continue;
		}
		std::wstring const subdir_name = it2->second;
		fs::wpath const src_dir_path = temp_dir_path/subdir_name;
		std::wstring file_title = it->getStringField(2);
		fs::wpath src_file_path = src_dir_path/file_title;
		if (!fs::exists(src_file_path)) {
			size_t ext_pos = file_title.rfind(L'.');
			// gqȂ
			if (ext_pos == std::wstring::npos) {
				file_title += L"._";
			} else {
				size_t const file_title_length = file_title.length();
				// gq2ȉ
				if (file_title_length - ext_pos - 1 <= 2) {
					file_title += L'_';
				} else {
					file_title[file_title_length - 1] = L'_';
				}
			}
			src_file_path = src_dir_path/file_title;
			// DOSNET.INFɗ񋓂Ăt@Cɂ݂͑Ȃ̂(̏ꍇXLbv)
			if (!fs::exists(src_file_path)) {
				continue;
			}
		}
		//
		const unsigned int field_count = it->getFieldCount();
		if (field_count >= 3) {
			fs::wpath const dest_file_path =
				this->work_dir_path_/subdir_name/it->getStringField(3);
			if (!fs::exists(dest_file_path)) {
				// 󂯑̃fBNg邩mFAȂ΍쐬
				fs::wpath dest_dir_path = dest_file_path;
				dest_dir_path.remove_filename();
				if (!fs::exists(dest_dir_path)) {
					fs::create_directories(dest_dir_path);
				}
				std::wstring const src_path_str = src_file_path.string();
				std::wstring const dest_path_str = dest_file_path.string();
				if (!::CopyFile(src_path_str.c_str(), dest_path_str.c_str(), FALSE)) {
					throw MY_WIN32_FILE_TRANSFER_EXCEPTION_LAST_ERROR(src_path_str, dest_path_str);
				}
			}
		}
		//
		fs::wpath const dest_file_path =
			this->work_dir_path_/subdir_name/file_title;
		if (!fs::exists(dest_file_path)) {
			moveFilesWithDirectoryCreation(src_file_path, dest_file_path);
		}
	}
	return true;
}

void SourceSizeReducer::moveDosnetDirectories(std::wstring const &section_name,
                                              boost::filesystem::wpath const &temp_dir_path)
{
	namespace fs = boost::filesystem;
	const MyLib::SetupFileLineIterator it_end;
	MyLib::SetupFileLineIterator it = this->dosnet_inf->findFirstLine(section_name.c_str());
	for (; it != it_end; ++it) {
		std::wstring const subdir_name = it->getStringField(1);
		if (subdir_name[0] == _T('\\')) {
			fs::wpath src_path = temp_dir_path/subdir_name;
			// DOSNET.INFɗ񋓂Ăt@Cɂ݂͑Ȃ̂(̏ꍇXLbv)
			if (fs::exists(src_path)) {
				fs::wpath const dest_path = this->work_dir_path_/subdir_name;
				moveFilesWithDirectoryCreation(src_path, dest_path);
			}
		} else {
			std::map<std::wstring, std::wstring, MyLib::LessNoCase>::const_iterator it2 =
				directories.begin();
			for (; it2 != directories.end(); ++it2) {
				fs::wpath const src_path = temp_dir_path/it2->second/subdir_name;
				// DOSNET.INFɗ񋓂Ăt@Cɂ݂͑Ȃ̂(̏ꍇXLbv)
				if (fs::exists(src_path)) {
					fs::wpath const dest_path = this->work_dir_path_/(it2->second)/subdir_name;
					moveFilesWithDirectoryCreation(src_path, dest_path);
				}
			}
		}
	}
}
