#include "USBSetup.h"
#include "mylib/function.h"
#include "mylib/SetupFileModifier.h"
#include "mylib/inifile.h"
#include "mylib/RegHive.h"
#include "FileCopyManager.h"
#include "mylib/storagedevicenumber.h"
#include "mylib/drivelayoutinformation.h"
#include "mylib/FileCompressionInfo.h"

extern void copyDirectory(boost::filesystem::wpath const &source_dir,
                          boost::filesystem::wpath const &destination_dir);


namespace {
/**
 * t@CEfBNg̈ړ(󂯑̃fBNgȂ΍쐬)
 */
void copyFilesWithDirectoryCreation(boost::filesystem::wpath const &src_path,
                                    boost::filesystem::wpath const &dest_path)
{
	namespace fs = boost::filesystem;
	fs::wpath dest_dir_path = dest_path;
	dest_dir_path.remove_filename();
	if (!fs::exists(dest_dir_path)) {
		fs::create_directories(dest_dir_path);
	}
	std::wstring const src_path_str = src_path.string();
	std::wstring const dest_path_str = dest_path.string();
	MyLib::toFileWritable(dest_path);
	if (!::CopyFile(src_path_str.c_str(), dest_path_str.c_str(), FALSE)) {
		throw MY_WIN32_FILE_TRANSFER_EXCEPTION_LAST_ERROR(src_path_str, dest_path_str);
	}
}

bool copyDosnetFiles(MyLib::SetupFileReader *dosnet_inf,
                     boost::filesystem::wpath const &work_dir_path,
                     std::wstring const &section_name,
                     boost::filesystem::wpath const &temp_dir_path,
                     std::map<std::wstring, std::wstring, MyLib::LessNoCase> const &directories)
{
	namespace fs = boost::filesystem;
	typedef std::map<std::wstring, std::wstring, MyLib::LessNoCase> MapStringToString;

	MyLib::SetupFileLineIterator const it_end;
	MyLib::SetupFileLineIterator it = dosnet_inf->findFirstLine(section_name.c_str());
	if (it == it_end) {
		return false;
	}
	for (; it != it_end; ++it) {
		MapStringToString::const_iterator it2 =
			directories.find(it->getStringField(1));
		if (it2 == directories.end()) {
			continue;
		}
		std::wstring const subdir_name = it2->second;
		fs::wpath const src_dir_path = temp_dir_path/subdir_name;
		std::wstring file_title = it->getStringField(2);
		fs::wpath src_file_path = src_dir_path/file_title;
		MyLib::FileCompressionInfo info(src_file_path);
		if (!info.exists()) {
			continue;
		}
		src_file_path = info.getActualPath();
		file_title = src_file_path.filename();
		//
		const unsigned int field_count = it->getFieldCount();
		if (field_count >= 3) {
			fs::wpath const dest_file_path =
				work_dir_path/it->getStringField(3);
			if (!fs::exists(dest_file_path)) {
				// 󂯑̃fBNg邩mFAȂ΍쐬
				fs::wpath dest_dir_path = dest_file_path;
				dest_dir_path.remove_filename();
				if (!fs::exists(dest_dir_path)) {
					fs::create_directories(dest_dir_path);
				}
				std::wstring const src_path_str = src_file_path.string();
				std::wstring const dest_path_str = dest_file_path.string();
				if (!::CopyFile(src_path_str.c_str(), dest_path_str.c_str(), FALSE)) {
					throw MY_WIN32_FILE_TRANSFER_EXCEPTION_LAST_ERROR(src_path_str, dest_path_str);
				}
			}
		}
		//
		fs::wpath const dest_file_path =
			work_dir_path/file_title;
		if (!fs::exists(dest_file_path)) {
			copyFilesWithDirectoryCreation(src_file_path, dest_file_path);
		}
	}
	return true;
}
/**
 * t@CRs[(ChJ[hgp)
 */
void copyFilesUsingWildCards(boost::filesystem::wpath const &src_path,
                             boost::filesystem::wpath const &dest_path,
                             std::wstring const &file_name)
{
	namespace fs = boost::filesystem;
	fs::wpath wild_card = src_path/file_name;
	WIN32_FIND_DATA fd;
	HANDLE h = ::FindFirstFile(wild_card.string().c_str(), &fd);
	if (h != INVALID_HANDLE_VALUE) {
		do {
			if (!(fd.cFileName[0] == _T('.') && (fd.cFileName[1] == _T('\0') || (fd.cFileName[1] == _T('.') && fd.cFileName[2] == _T('\0'))))) {
				fs::wpath const src_file_path = src_path/fd.cFileName;
				fs::wpath const dest_file_path = dest_path/fd.cFileName;
				MyLib::toFileWritable(src_path);
				if (!::CopyFile(src_file_path.string().c_str(), dest_file_path.string().c_str(), FALSE)) {
					::FindClose(h);
					throw MY_WIN32_FILE_TRANSFER_EXCEPTION_LAST_ERROR(src_file_path.string(), dest_file_path.string());
				}
			}
		} while (::FindNextFile(h, &fd));
		::FindClose(h);
	}
}

/////////////////////////////////////////////////////////////////////////////
// boot image
/////////////////////////////////////////////////////////////////////////////

static size_t const BOOT_IMAGE_SIZE = 512;///

bool isBootImage(unsigned char const *ptr0, char const *str)
{
	// think of boot image if contains "CDBOOT:"
	//static char const *CDBOOT = "NTLDR is missing";///
	size_t const str_len = strlen(str);
	
	size_t search_range = BOOT_IMAGE_SIZE;
	unsigned char const *ptr =
		reinterpret_cast<unsigned char const*>(std::memchr(ptr0, str[0], search_range));
	while (ptr != NULL) {
		if (std::memcmp(ptr, str, str_len) == 0) {
			return true;
		}
		search_range = BOOT_IMAGE_SIZE - (ptr - ptr0 + 1);
		ptr = reinterpret_cast<unsigned char const*>(std::memchr(ptr + 1, str[0], search_range));
	}
	return false;
}

unsigned char const *getBootImageStartPoint(unsigned char *buf, size_t buf_size)
{
	if (buf_size > BOOT_IMAGE_SIZE) {
		static unsigned char const SIGNATURE[] = {0x55, 0xAA};
		static size_t const SIGNATURE_SIZE = _countof(SIGNATURE);

		unsigned char const *ptr = NULL;
		size_t const search_range0 = buf_size - (BOOT_IMAGE_SIZE - SIGNATURE_SIZE);
		size_t search_range = search_range0;
		ptr = reinterpret_cast<unsigned char const*>(std::memchr(buf + (BOOT_IMAGE_SIZE - SIGNATURE_SIZE), SIGNATURE[0], search_range));
		while (ptr != NULL) {
			size_t n1 = ptr - buf + 1;
			int nn1 = n1 % 16;
			if (ptr[1] == SIGNATURE[1] && nn1 == 15) {
				unsigned char const *ptr1 = ptr - (BOOT_IMAGE_SIZE - SIGNATURE_SIZE);
				if (isBootImage(ptr1, "FAT12") && isBootImage(ptr1, "NTLDR is missing")) {
					return ptr1;
				}
			}
			search_range = search_range0 + n1;
			ptr = reinterpret_cast<unsigned char const*>(std::memchr(ptr + 1, SIGNATURE[0], search_range));
		}
	}
	return NULL;
}

unsigned char const *getNTFSBootImageStartPoint(unsigned char *buf, size_t buf_size)
{
	if (buf_size > 8192) {
		static unsigned char const SIGNATURE[] = {0x55, 0xAA, 0x05, 0x00, 0x4E, 0x00, 0x54, 0x00, 0x4C, 0x00, 0x44, 0x00, 0x52, 0x00};
		static size_t const SIGNATURE_SIZE = _countof(SIGNATURE);

		unsigned char const *ptr = NULL;
		size_t const search_range0 = buf_size - 8192;
		size_t search_range = search_range0;
		ptr = reinterpret_cast<unsigned char const*>(std::memchr(buf + (BOOT_IMAGE_SIZE - 2), SIGNATURE[0], search_range));
		while (ptr != NULL) {
			size_t n1 = ptr - buf + 1;
			//int nn1 = n1 % 16;
			if (std::memcmp(SIGNATURE, ptr, SIGNATURE_SIZE) == 0) {
				unsigned char const *ptr1 = ptr - (BOOT_IMAGE_SIZE - 2);
				if (isBootImage(ptr1, "Press Ctrl+Alt+Del to restart")) {
					return ptr1;
				}
			}
			search_range = search_range0 + n1;
			ptr = reinterpret_cast<unsigned char const*>(std::memchr(ptr + 1, SIGNATURE[0], search_range));
		}
	}
	return NULL;
}

//bool hasPartittionTable(unsigned char *buf)
//{
//	if (*buf != 0xEB) {
//		int bt_flag_count = 0;
//		int no_bt_flag_count = 0;
//		for (unsigned int i = 0x01BE; i <= 0x01EE; i += 0x10) {
//			if (buf[i] == 0x80) {
//				++bt_flag_count;
//			} else if (buf[i] == 0) {
//				++no_bt_flag_count;
//			}
//		}
//		if ((bt_flag_count == 1 && no_bt_flag_count == 3)
//		 || (bt_flag_count == 0 && no_bt_flag_count == 4)) {
//			return true;
//		}
//	}
//	return false;
//}

unsigned char const *getMBRStartPoint(unsigned char *buf, size_t buf_size)
{
	if (buf_size > BOOT_IMAGE_SIZE) {
		static unsigned char const SIGNATURE[] = {0x55, 0xAA};
		static size_t const SIGNATURE_SIZE = _countof(SIGNATURE);

		unsigned char const *ptr = NULL;
		size_t const search_range0 = buf_size - (BOOT_IMAGE_SIZE - SIGNATURE_SIZE);
		size_t search_range = search_range0;
		ptr = reinterpret_cast<unsigned char const*>(std::memchr(buf + (BOOT_IMAGE_SIZE - SIGNATURE_SIZE), SIGNATURE[0], search_range));
		while (ptr != NULL) {
			size_t n1 = ptr - buf + 1;
			int nn1 = n1 % 16;
			if (ptr[1] == SIGNATURE[1] && nn1 == 15) {
				unsigned char const *ptr1 = ptr - (BOOT_IMAGE_SIZE - SIGNATURE_SIZE);
				if (isBootImage(ptr1, "Invalid partition table")) {
					return ptr1;
				}
			}
			search_range = search_range0 + n1;
			ptr = reinterpret_cast<unsigned char const*>(std::memchr(ptr + 1, SIGNATURE[0], search_range));
		}
	}
	return NULL;
}


// NTLDR ̃o[Wr
// (OSCXg[ɌǒuĂ܂߈ӖȂ)
//uint64_t getNTLDRVersion(boost::filesystem::wpath const ntldr_path)
//{
//	size_t const s = fs::file_size(ntldr_path);
//	fs::ifstream ifs(ntldr_path, std::ios::in | std::ios::binary);
//	if (!ifs) {
//		throw MY_STD_C_FILE_EXCEPTION(ntldr_path.string(), EINVAL);
//	}
//	// t@CŜobt@ɓǂݍ
//	boost::scoped_array<unsigned char> ntldr_buf(new unsigned char[s]);
//	ifs.read(reinterpret_cast<char*>(ntldr_buf.get()), s);
//	// u[gC[W̕TAt@Cɕۑ
//
//	static wchar_t const *SIGNATURE = L"VS_VERSION_INFO";
//	static size_t const SIGNATURE_SIZE = wcslen(SIGNATURE)*sizeof(wchar_t);
//	unsigned char const *ptr0 = ntldr_buf.get();
//	size_t search_range = s;
//	unsigned char const *ptr =
//		reinterpret_cast<unsigned char const*>(std::memchr(ptr0, 'V', search_range));
//	while (ptr != NULL) {
//		search_range = s - (ptr - ptr0 + 1);
//		if (search_range > 54 && std::memcmp(ptr, SIGNATURE, SIGNATURE_SIZE) == 0) {
//			uint64_t const *ver_ptr = reinterpret_cast<uint64_t const*>(ptr + 46);
//			ULARGE_INTEGER uli;
//			uli.QuadPart = *ver_ptr;
//			if (0x00050000 <= uli.HighPart && uli.HighPart < 0x00060000) {
//				return *ver_ptr;
//			}
//		}
//		ptr = reinterpret_cast<unsigned char const*>(std::memchr(ptr + 1, 'V', search_range));
//	}
//	return 0;
//}
};

/////////////////////////////////////////////////////////////////////////////

USBSetup::USBSetup(boost::filesystem::wpath const &work_dir_path,
                   std::wstring const &i386_subdir_name,
                   uint32_t os_ver,
                   wchar_t target_drive_letter,
                   wchar_t setup_drive_letter)
: work_dir_path(work_dir_path)
, i386_subdir_name(i386_subdir_name)
, os_ver(os_ver)
, target_drive_letter(target_drive_letter)
, setup_drive_letter(setup_drive_letter)
{
}

USBSetup::~USBSetup()
{
}

void USBSetup::execute()
{
	namespace fs = boost::filesystem;

	wchar_t root_path[] = {this->target_drive_letter, ':', '\\', '\0'};
	fs::wpath target_drive_root = root_path;
	// Workdir -> USB
	this->fireEvent(L"USBփt@C̃Rs[BBB");
	copyDirectory(this->work_dir_path, target_drive_root);
	this->fireEvent(L"\n$WIN_NT$.~BT tH_쐬BBB");
	// $WIN_NT$.~BT
	fs::wpath bt_dir_path = target_drive_root/L"$WIN_NT$.~BT";
	fs::create_directory(bt_dir_path);
	// DOSNET.INFJ
	fs::wpath const dosnet_inf_path = this->work_dir_path/this->i386_subdir_name/L"DOSNET.INF";
	MyLib::SetupFileReader dosnet_inf(dosnet_inf_path.string());
	// DOSNET.INF[Directories]ZNVAfBNgꗗ쐬
	MyLib::SetupFileLineIterator const it_end;
	std::map<std::wstring, std::wstring, MyLib::LessNoCase> directories;
	MyLib::SetupFileLineIterator it = dosnet_inf.findFirstLine(L"Directories");
	if (this->os_ver == 0x00050000) {
		for (; it != it_end; ++it) {
			std::wstring const dir_name = L"\\" + this->i386_subdir_name + it->getStringField(1);
			directories[it->getStringField(0)] = dir_name;
		}
	} else {
		for (; it != it_end; ++it) {
			std::wstring const dir_name = it->getStringField(1);
			// cmpnentsfBNgO
			if (MyLib::compareStringNoCase(dir_name.substr(0, 10), L"\\cmpnents\\") != 0) {
				directories[it->getStringField(0)] = dir_name;
			}
		}
	}
	// DOSNET.INF[FloppyFiles.*]ZNVɗ񋓂Ăt@C߂
	int i = 0;
	std::wostringstream ss;
	do {
		ss.str(_T(""));
		ss << _T("FloppyFiles.") << i;
		++i;
	} while (copyDosnetFiles(&dosnet_inf, bt_dir_path, ss.str(), this->work_dir_path, directories));
	this->fireEvent(L"\nu[gZN^̏ݒBBB");
	// t@CVXe𒲂ׂ
	wchar_t file_system_name[MAX_PATH];
	BOOL ret = ::GetVolumeInformation(target_drive_root.string().c_str(), NULL, 0, NULL, NULL, NULL, file_system_name, MAX_PATH);
	if (!ret) {
		throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(target_drive_root.string());
	}
	enum FileSystemID {
		FAT,
		//FAT32,
		NTFS,
		OTHERS
	};
	FileSystemID fs_id = OTHERS;
	if (wcscmp(file_system_name, L"FAT") == 0) {
		fs_id = FAT;
	//} else if (wcscmp(file_system_name, L"FAT32") == 0) {
	//	fs_id = FAT32;
	} else if (wcscmp(file_system_name, L"NTFS") == 0) {
		fs_id = NTFS;
	}
	if (fs_id == OTHERS) {
		throw std::runtime_error("This file system is not supported.");
	}

	//std::wstring ntldr_name;
	//std::wstring setupldr_name;
	wchar_t buf[256];
	// {[J
	::wsprintf(buf, L"\\\\.\\%c:", this->target_drive_letter);
	MyLib::AutoHandleI h = ::CreateFile(buf,
		GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
		OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (h.isInvalid()) {
		throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(buf);
	}
	// u[gZN^ǂݍ
	unsigned int const bootsect_size = ((fs_id == NTFS) ? 8192 : 512);
	boost::scoped_array<unsigned char> bootsect_buf(new unsigned char[bootsect_size]);
	DWORD read_bytes;
	BOOL ret1 = ::ReadFile(h.get(), bootsect_buf.get(), bootsect_size, &read_bytes, NULL);
	if (!ret1 || read_bytes != bootsect_size) {
		throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(buf);
	}
	// BOOTSECT.DAT 쐬
	{
	// SPCMDCON.SYS u[gC[W𔲂o
	fs::wpath const spcmdcon_sys_path = this->work_dir_path/this->i386_subdir_name/L"SPCMDCON.SYS";
	fs::ifstream ifs(spcmdcon_sys_path, std::ios::in | std::ios::binary);
	if (!ifs) {
		throw MY_STD_C_FILE_EXCEPTION(spcmdcon_sys_path.string(), EINVAL);
	}
	// t@CŜobt@ɓǂݍ
	size_t const s = ifs.seekg(0, std::ios::end).tellg();
	ifs.seekg(0, std::ios::beg);
	boost::scoped_array<unsigned char> spcmdcon_buf(new unsigned char[s]);
	ifs.read(reinterpret_cast<char*>(spcmdcon_buf.get()), s);
	if (fs_id == NTFS) {
		// u[gC[W̕TAt@Cɕۑ
		unsigned char const *ptr = getNTFSBootImageStartPoint(spcmdcon_buf.get(), s);
		if (ptr == NULL) {
			throw MY_STD_C_FILE_EXCEPTION(spcmdcon_sys_path.string(), EINVAL);
		}
		// u[gZN^IPL SPCMDCON.SYS ̃u[gC[Ŵ̂ɏ
		if (::SetFilePointer(h.get(), 0, NULL, FILE_BEGIN) == INVALID_SET_FILE_POINTER) {
			throw MY_WIN32_FILE_EXCEPTION(buf, INVALID_SET_FILE_POINTER);
		}
		std::memcpy(bootsect_buf.get(), ptr, 3);
		std::memcpy(bootsect_buf.get() + 0x54, ptr + 0x54, 8108);
		DWORD written_bytes;
		BOOL ret12 = ::WriteFile(h.get(), bootsect_buf.get(), bootsect_size, &written_bytes, NULL);
		if (!ret12 || written_bytes != bootsect_size) {
			throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(buf);
		}
		// 
		bootsect_buf[0x0202] = L'S';
	} else {
		// u[gC[W̕TAt@Cɕۑ
		unsigned char const *ptr = getBootImageStartPoint(spcmdcon_buf.get(), s);
		if (ptr == NULL) {
			throw MY_STD_C_FILE_EXCEPTION(spcmdcon_sys_path.string(), EINVAL);
		}
		// u[gZN^IPL SPCMDCON.SYS ̃u[gC[Ŵ̂ɏ
		if (::SetFilePointer(h.get(), 0, NULL, FILE_BEGIN) == INVALID_SET_FILE_POINTER) {
			throw MY_WIN32_FILE_EXCEPTION(buf, INVALID_SET_FILE_POINTER);
		}
		std::memcpy(bootsect_buf.get(), ptr, 3);
		std::memcpy(bootsect_buf.get() + 0x3E, ptr + 0x3E, 448);
		bootsect_buf[0x24] = 0x80;  // 0x00 ɂȂĂ Disk error
		DWORD written_bytes;
		BOOL ret12 = ::WriteFile(h.get(), bootsect_buf.get(), bootsect_size, &written_bytes, NULL);
		if (!ret12 || written_bytes != bootsect_size) {
			throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(buf);
		}
		// 
		static char const *original_ldr_name = "NTLDR      ";
		static unsigned int const ldr_name_len = static_cast<unsigned int>(strlen(original_ldr_name));
		unsigned int i = 0;
		for (; i < bootsect_size - ldr_name_len; ++i) {
			if (bootsect_buf[i] == original_ldr_name[0]) {
			if (::memcmp(bootsect_buf.get() + i, original_ldr_name, ldr_name_len) == 0) {
				break;
			}
			}
		}
		bootsect_buf[i] = L'S';
	}
	}
	{
		fs::wpath bootsect_dat_path = bt_dir_path/L"BOOTSECT.DAT";
		fs::ofstream ofs(bootsect_dat_path, std::ios::out | std::ios::binary);
		if (!ofs) {
			throw MY_WIN32_FILE_EXCEPTION(bootsect_dat_path.string(), ERROR_FILE_NOT_FOUND);
		}
		ofs.write(reinterpret_cast<char*>(bootsect_buf.get()), bootsect_size);
	}

	// [g ntldr, ntdetect.com, bootfont.bin, boot.ini,
	//          txtsetup.sif, stldr(setupldr.bin) 
	FileCopyManager i386dir_to_root(this->work_dir_path/L"I386", target_drive_root);
	// NTLDR
	i386dir_to_root.copyFile(L"NTLDR");
	// NTDETECT.COM
	i386dir_to_root.copyFile(L"NTDETECT.COM");
	// SETUPLDR.BIN
	i386dir_to_root.copyFile(L"SETUPLDR.BIN", L"STLDR" /*setupldr_name*/);
	// SETUPREG.HIV
	{
		wchar_t buf1[MAX_PATH];
		::wsprintf(buf1, L"\\DosDevices\\%c:", this->target_drive_letter);
		CRegKey reg1;
		reg1.Open(HKEY_LOCAL_MACHINE, L"SYSTEM\\MountedDevices");
		DWORD reg_value_size = 0;
		::RegQueryValueEx(reg1, buf1, NULL, NULL, NULL, &reg_value_size);
		boost::scoped_array<BYTE> buf2(new BYTE[reg_value_size]);
		::RegQueryValueEx(reg1, buf1, NULL, NULL, buf2.get(), &reg_value_size);
		//wchar_t buf3[MAX_PATH];
		//DWORD s4 = MAX_PATH;
		//DWORD s5 = 0;
		//std::wstring volume_guid_name;
		//for (unsigned int i = 0; ::RegEnumValue(reg1, i, buf3, &s4, NULL, NULL, NULL, &s5) == 0; ++i) {
		//	if (wcsncmp(buf3, L"\\??\\", 4) == 0 && s5 == reg_value_size) {
		//		boost::scoped_array<BYTE> buf6(new BYTE[reg_value_size]);
		//		::RegQueryValueEx(reg1, buf3, NULL, NULL, buf6.get(), &s5);
		//		if (::memcmp(buf2.get(), buf6.get(), reg_value_size) == 0) {
		//			volume_guid_name = buf3;
		//			break;
		//		}
		//	}
		//	s4 = MAX_PATH;
		//}

		fs::wpath setupreg_hiv_path = bt_dir_path/L"SETUPREG.HIV";
		MyLib::RegHive setupreg_hiv(setupreg_hiv_path.string());
		CRegKey reg;
		HRESULT hres1 = reg.Create(setupreg_hiv, _T("MountedDevices"));
		if (FAILED(hres1)) throw MY_WIN32_EXCEPTION(hres1);
		wchar_t buf7[MAX_PATH];
		::wsprintf(buf7, L"\\DosDevices\\%c:", this->setup_drive_letter);
		LSTATUS ret = ::RegSetValueEx(reg, buf7, NULL, REG_BINARY, buf2.get(), reg_value_size);
		if (ret != ERROR_SUCCESS) throw MY_WIN32_EXCEPTION(ret);
		//ret = ::RegSetValueEx(reg, volume_guid_name.c_str(), NULL, REG_BINARY, buf2.get(), reg_value_size);
		//if (ret != ERROR_SUCCESS) throw MY_WIN32_EXCEPTION(ret);
	}
	// TXTSETUP.SIF
	FileCopyManager i386_to_root(this->work_dir_path/this->i386_subdir_name, target_drive_root);
	i386_to_root.copyFile(L"TXTSETUP.SIF");
	// txtsetup.sif [SetupData] SetupSourceDevice
	fs::wpath txtsetup_sif_path = target_drive_root/L"TXTSETUP.SIF";
	wchar_t buf2[256];
	::wsprintf(buf2, L"\\DosDevices\\%c:", this->setup_drive_letter);
	MyLib::toFileWritable(txtsetup_sif_path);
	IniFile txtsetup_sif(txtsetup_sif_path);
	txtsetup_sif.setString(L"SetupData", L"SetupSourceDevice", buf2);

	FileCopyManager bin_to_root(*BIN_DIR_PATH, target_drive_root);
	// $WIN_NT$.~BT\WINNT.SIF
	bin_to_root.copyFile(L"ldrfix.exe");

	FileCopyManager i386_to_bt(this->work_dir_path/this->i386_subdir_name, bt_dir_path);
	i386_to_bt.copyFilesUsingWildCards(L"WINNT.SIF");
	fs::wpath winnt_sif_path = bt_dir_path/L"WINNT.SIF";
	MyLib::toFileWritable(winnt_sif_path);
	IniFile winnt_sif(winnt_sif_path);
	winnt_sif.setString(L"Data", L"UnattendedInstall", L"Yes");
	winnt_sif.setString(L"Data", L"MSDosInitiated", L"No");
	winnt_sif.setInt(L"Data", L"EulaComplete", 1);
	{
		MyLib::SetupFileModifier winnt_sif2(winnt_sif_path.string());
		std::wstring str;
		str += this->setup_drive_letter;
		str += L":\\ldrfix.exe";
		winnt_sif2.insertLine(L"GuiRunOnce", str);
		winnt_sif2.flush();
	}

	/////////////////////////////////////////////////////////////////////////
	// boot.ini
	/////////////////////////////////////////////////////////////////////////
	MyLib::StorageDeviceNumber sdn(this->target_drive_letter);
	//STORAGE_DEVICE_NUMBER sdn;
	//DWORD junk;
	//ret = DeviceIoControl(h.get(), IOCTL_STORAGE_GET_DEVICE_NUMBER, NULL, 0,
	//	&sdn, sizeof(STORAGE_DEVICE_NUMBER), &junk, NULL);
	//if (!ret) {
	//	throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(buf);
	//}
	////FILE_DEVICE_DISK
	//wchar_t physical_drive_name[32];
	//wsprintf(physical_drive_name, L"\\\\.\\PhysicalDrive%u", sdn.getDeviceNumber());
	//MyLib::AutoHandle<INVALID_HANDLE_VALUE> h2 = ::CreateFile(
	//	physical_drive_name, GENERIC_READ | GENERIC_WRITE,
	//	FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
	//	FILE_ATTRIBUTE_NORMAL, NULL);
	//if (h2.isInvalid()) {
	//	throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(physical_drive_name);
	//}
	// MBR ̓ǂݍ݂Ə
	// TODO: p[eBV΍
	{
		wchar_t physical_drive_name[32];
		wsprintf(physical_drive_name, L"\\\\.\\PhysicalDrive%u", sdn.getDeviceNumber());
		MyLib::AutoHandleI h = ::CreateFile(
			physical_drive_name, GENERIC_READ | GENERIC_WRITE,
			FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
			FILE_ATTRIBUTE_NORMAL, NULL);
		if (h.isInvalid()) {
			throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(physical_drive_name);
		}
		// u[gZN^ǂݍ
		static unsigned int const BOOTSECT_SIZE = 512;
		boost::scoped_array<char> bootsect_buf(new char[BOOTSECT_SIZE]);
		DWORD read_bytes;
		BOOL ret1 = ::ReadFile(h.get(), bootsect_buf.get(), BOOTSECT_SIZE, &read_bytes, NULL);
		if (!ret1 || read_bytes != BOOTSECT_SIZE) {
			throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(physical_drive_name);
		}
		// t@C|C^擪ɖ߂
		if (::SetFilePointer(h.get(), 0, NULL, FILE_BEGIN) == INVALID_SET_FILE_POINTER) {
			throw MY_WIN32_FILE_EXCEPTION(buf, INVALID_SET_FILE_POINTER);
		}
		// SPCMDCON.SYS u[gC[W𔲂o
		fs::wpath const spcmdcon_sys_path = this->work_dir_path/this->i386_subdir_name/L"SPCMDCON.SYS";
		fs::ifstream ifs(spcmdcon_sys_path, std::ios::in | std::ios::binary);
		if (!ifs) {
			throw MY_STD_C_FILE_EXCEPTION(spcmdcon_sys_path.string(), EINVAL);
		}
		// t@CŜobt@ɓǂݍ
		size_t const s = ifs.seekg(0, std::ios::end).tellg();
		ifs.seekg(0, std::ios::beg);
		boost::scoped_array<unsigned char> spcmdcon_buf(new unsigned char[s]);
		ifs.read(reinterpret_cast<char*>(spcmdcon_buf.get()), s);
		// u[gC[W̕TAt@Cɕۑ
		unsigned char const *ptr = getMBRStartPoint(spcmdcon_buf.get(), s);
		if (ptr == NULL) {
			throw MY_STD_C_FILE_EXCEPTION(spcmdcon_sys_path.string(), EINVAL);
		}
		// u[gZN^IPL SPCMDCON.SYS ̃u[gC[Ŵ̂ɏ
		std::memcpy(bootsect_buf.get(), ptr, 440);
		DWORD written_bytes;
		BOOL ret12 = ::WriteFile(h.get(), bootsect_buf.get(), bootsect_size, &written_bytes, NULL);
		if (!ret12 || written_bytes != bootsect_size) {
			throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(buf);
		}
	}
	// fBXÑp[eBV\擾
	MyLib::DriveLayoutInformation dli(sdn.getDeviceNumber());
	//unsigned int n = 1;
	//DWORD bytes_returned;
	//DRIVE_LAYOUT_INFORMATION *layout_info = (DRIVE_LAYOUT_INFORMATION*)HeapAlloc(GetProcessHeap(),
	//                                                   HEAP_ZERO_MEMORY,
	//                                                   DRIVE_LAYOUT_INFO_SIZE(n));
	//for (;;) {
	//	ret = DeviceIoControl(h2.get(),
	//	                      IOCTL_DISK_GET_DRIVE_LAYOUT,
	//	                      NULL,
	//	                      0,
	//	                      layout_info,
	//	                      DRIVE_LAYOUT_INFO_SIZE(n),
	//	                      &bytes_returned,
	//	                      NULL);
	//	if (ret) {
	//		break;
	//	} else {
	//		if (GetLastError() == ERROR_INSUFFICIENT_BUFFER) {
	//			++n;
	//			layout_info = (DRIVE_LAYOUT_INFORMATION*)HeapReAlloc(GetProcessHeap(),
	//			                                                     HEAP_ZERO_MEMORY,
	//			                                                     layout_info,
	//			                                                     DRIVE_LAYOUT_INFO_SIZE(n));
	//		} else {
	//			HeapFree(GetProcessHeap(), 0, layout_info);
	//			// throw
	//		}
	//	}
	//}
	//unsigned int layout_info_size = bytes_returned;

	int active_index = dli.getActivePartition();
	int target_index = -1;
	for (int i = 0; i < min(static_cast<int>(dli.getPartitionCount()), 4); ++i) {
		PARTITION_INFORMATION const *pi = dli.getPartitionEntry(i);
		if (pi->PartitionNumber == sdn.getPartitionNumber()) {
			target_index = i;
		}
	}
	//{
	//fs::ofstream ofs("D:\\test.txt");
	//PARTITION_INFORMATION const *ap = dli.getPartitionEntry(active_index);
	//PARTITION_INFORMATION const *tp = dli.getPartitionEntry(target_index);
	//ofs << "active : " << active_index << ", " << ap->PartitionNumber << ", " << static_cast<unsigned int>(ap->PartitionType) << ", " << ap->BootIndicator << ", " << ap->RecognizedPartition << std::endl;
	//ofs << "active : " << target_index << ", " << tp->PartitionNumber << ", " << static_cast<unsigned int>(tp->PartitionType) << ", " << tp->BootIndicator << ", " << tp->RecognizedPartition << std::endl;
	//}
	if (target_index < 0 || 4 <= target_index || !dli.isRecognizedPartition(target_index)) {
		throw std::runtime_error("The specified drive is not a basic partition.");
	}

	if (target_index != active_index) {
		// ZbgAbvhCũp[eBVANeBuɂ
		//if (active_index != 0xFFFFFFFF) {
		//	layout_info->PartitionEntry[active_index].BootIndicator = false;
		//	layout_info->PartitionEntry[active_index].RewritePartition = true;
		//}
		//layout_info->PartitionEntry[target_index].BootIndicator = true;
		//layout_info->PartitionEntry[target_index].RewritePartition = true;
		//ret = DeviceIoControl(h2.get(), IOCTL_DISK_SET_DRIVE_LAYOUT, layout_info, layout_info_size, NULL, 0, &bytes_returned, NULL);
		//if (!ret) {
		//	throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(physical_drive_name);
		//}
		dli.setActivePartiton(target_index);
		// copy boot.ini from active partition
		if (0 <= active_index && active_index < 4 && dli.isRecognizedPartition(active_index)) {
			wchar_t src_path[MAX_PATH];
			wsprintf(src_path,
					 L"\\\\?\\GLOBALROOT\\Device\\Harddisk%u\\Partition%u",
					 sdn.getDeviceNumber(),
					 dli.getPartitionEntry(active_index)->PartitionNumber);
			FileCopyManager boot_ini_copy(src_path, target_drive_root);
			//boot_ini_copy.copyFilesUsingWildCards(L"boot.ini");
			boot_ini_copy.copyFile(L"boot.ini");
		}
	}
	// boot.ini ɒǋL
	fs::wpath const boot_ini_path = target_drive_root/L"boot.ini";
	MyLib::toFileWritable(boot_ini_path);
	IniFile boot_ini(boot_ini_path);
	boot_ini.setInt(L"boot loader", L"timeout", 30);
	boot_ini.setString(L"operating systems", L"C:\\$WIN_NT$.~BT\\BOOTSECT.DAT", L"\"Windows ZbgAbv Jn\"");
	// XP/2003 ł GUI Setup Start
	if (this->os_ver >= 0x00050001/* && dli.getPartitionTableEntryCount() > 1*/) {
		boot_ini.setString(L"boot loader", L"default", L"multi(0)disk(0)rdisk(1)partition(1)\\WINDOWS");
		boot_ini.setString(L"operating systems", L"multi(0)disk(0)rdisk(1)partition(1)\\WINDOWS", L"\"Windows ZbgAbv iK\" /FASTDETECT");
	} else {
		boot_ini.setString(L"boot loader", L"default", L"C:\\$WIN_NT$.~BT\\BOOTSECT.DAT");
	}
	FileCopyManager boot_ini_backup(target_drive_root, target_drive_root);
	boot_ini_backup.copyFile(L"boot.ini", L"bootorig.ini");
	this->fireEvent(L"\n");
}
