#ifndef SETUP_FILE_PARSER_H__FAD9809C_AE84_4D3C_A4C3_0F775A801BD7__INCLUDED
#define SETUP_FILE_PARSER_H__FAD9809C_AE84_4D3C_A4C3_0F775A801BD7__INCLUDED
// M$ compatible compilers support #pragma once
#if defined(_MSC_VER) && (_MSC_VER >= 1020)
#pragma once
#endif

/////////////////////////////////////////////////////////////////////////////
// SetupFileParser.h
//  H20(2008)/03/06 쐬
// Classes in this file:
//  SetupFileLine
//  SetupFileSection
//  TextIteratorBase
//  CP932Iterator
//  UTF16Iterator
//  SetupFileParser
/////////////////////////////////////////////////////////////////////////////

#include "exception.h"
#include "function.h"

namespace MyLib {
typedef size_t FilePointerType;

//=======================================================================
// SetupFileLine
//  ZbgAbvt@C̃ZNV̈s
//=======================================================================
class SetupFileLine
{
public:
	typedef std::map<unsigned, std::wstring> FieldCollection;
	typedef std::pair<FilePointerType, FilePointerType> RangeType;
	// RXgN^
	SetupFileLine(FilePointerType start_pos = 0)
		: field_count(0)
		, range(start_pos, start_pos)
	{
	}
	// s̏Iʒuݒ肷B
	void setEndPosition(FilePointerType end_position)
	{
		this->range.second = end_position;
	}
	// s͈̔͂擾B
	RangeType getRange() const
	{
		return range;
	}
	// tB[h̒ǉ
	void addField(unsigned field_index, const std::wstring& field_string)
	{
		if (this->field_count < field_index) {
			this->field_count = field_index;
		}
		// 󕶎͋L^Ȃ
		if (!field_string.empty()) {
			this->fields.insert(std::make_pair(field_index, field_string));
		}
	}
	// tB[h擾
	unsigned getFieldCount() const
	{
		return field_count;
	}
	// w肳ꂽtB[h琮l擾
	int getIntField(unsigned field_index) const
	{
		const std::wstring str = this->getStringField(field_index);
		const std::wstring::size_type length = str.length();
		LPCTSTR start_ptr;
		LPTSTR stop_ptr;
		// ܂A10iւ̕ϊ݂
		int result = ::_tcstol((start_ptr = str.c_str()), &stop_ptr, 10);
		// ɂAϊ̏Iʒu擾
		const std::wstring::size_type stop_pos = stop_ptr - start_ptr;
		assert(0 <= stop_pos && stop_pos <= length);
		// ̍Ō܂ŕϊĂȂꍇ
		if (stop_pos != length) {
			if (str[stop_pos] == 'x' || str[stop_pos] == 'X') {
				// 16iւ̕ϊ݂
				result = ::_tcstol((start_ptr = str.c_str()), &stop_ptr, 16);
				const std::wstring::size_type stop_pos = stop_ptr - start_ptr;
				if (stop_pos != length) {
					//throw Win32Error(ERROR_INVALID_DATA);
					throw MY_WIN32_EXCEPTION(ERROR_INVALID_DATA);
				}
			} else {
				//throw Win32Error(ERROR_INVALID_DATA);
				throw MY_WIN32_EXCEPTION(ERROR_INVALID_DATA);
			}
		}
		return result;
	}
	// w肳ꂽtB[h̕擾
	std::wstring getStringField(unsigned field_index) const
	{
		std::wstring temp;
		FieldCollection::const_iterator it = fields.find(field_index);
		if (it == fields.end()) {
			if (field_index == 0 || field_index > this->getFieldCount()) {
				//throw Win32Error(ERROR_INVALID_PARAMETER);
				throw MY_WIN32_EXCEPTION(ERROR_INVALID_PARAMETER);
			}
		} else {
			temp = it->second;
		}
		return temp;
	}
	// L[邩H
	bool hasKey(void) const
	{
		FieldCollection::const_iterator it = fields.find(0);
		return it != fields.end();
	}

private:
	// tB[h
	unsigned field_count;
	// tB[h̕
	FieldCollection fields;
	// t@C̍s͈̔(JnEIʒu)
	RangeType range;
};

//=======================================================================
// SetupFileSection
//  ZbgAbvt@C̈̃ZNV
//=======================================================================
struct SetupFileSection
{
	typedef std::list<SetupFileLine> SetupFileLineCollection;
	typedef std::vector<SetupFileLine::RangeType> RangeCollection;
	// ZNV̍s
	SetupFileLineCollection lines;
	// ZNV̊JnEI|C^
	RangeCollection ranges;
};

class TextIteratorBase
	: public std::iterator<std::input_iterator_tag, char>
{
protected:
	typedef TextIteratorBase this_type;
	// f[^̐擪|C^
	const char *start_ptr;
	// ݂̈ʒu(ꂩǂݏoʒu)
	size_t current_pos;
	// I[
	size_t eof_pos;
	// ǂݏoꕶ
	int32_t value;

public:
	// ftHgRXgN^
	TextIteratorBase(void)
		: start_ptr(NULL)
		, current_pos(0)
		, eof_pos(0)
		, value(-1)
	{
	}
	TextIteratorBase(void *data_ptr, size_t file_size)
		: start_ptr(reinterpret_cast<const char*>(data_ptr))
		, current_pos(0)
		, eof_pos(file_size)
	{
	}
	// ǂ݂ƂꕶԂB
	int32_t operator*(void) const
	{
		return value;
	}
	// ݂̈ʒuԂB
	size_t tellg(void)
	{
		return current_pos;
	}
	// rZq
	bool operator==(const this_type& rhs) const
	{
		return this->start_ptr == rhs.start_ptr
			&& this->current_pos == rhs.current_pos
			&& this->eof_pos == rhs.eof_pos;
	}
	bool operator!=(const this_type& rhs) const
	{
		return !(*this == rhs);
	}
};

//=======================================================================
// CP932Iterator
//  z񂩂SJISGR[h(CP932)1ÂCe[^
//=======================================================================
class CP932Iterator
	: public TextIteratorBase
{
private:
	// 1oCgǂݏoA|C^i߂
	int32_t get(void)
	{
		int32_t temp = (this->current_pos < this->eof_pos ?
			*(this->start_ptr + this->current_pos) : EOF);
		++this->current_pos;
		return temp;
	}

public:
	// ftHgRXgN^
	CP932Iterator(void)
	{
	}
	// RXgN^
	CP932Iterator(void *data_ptr, size_t file_size)
		: TextIteratorBase(data_ptr, file_size)
	{
		if (file_size > 0) {
			++*this;
		} else {
			this->start_ptr = NULL;
		}
	}
	// OuCNgZq
	const this_type& operator++(void)
	{
		// 1oCgڂǂݏo
		int32_t first_byte = this->get();
		if (first_byte != EOF && first_byte != 0x1A) {
			// 2oCg̐擪oCgoꂽ
			if (::_ismbblead(first_byte)) {
				// 2oCgڂǂݏo
				int32_t second_byte = this->get();
				// 2oCg
				if (second_byte != EOF && ::_ismbbtrail(second_byte)) {
					// Unicodeϊ
					// Surrogate͍lĂȂ(SJIS̕ϊvH)
					char str[] = {
						static_cast<char>(first_byte),
						static_cast<char>(second_byte)
					};
					::MultiByteToWideChar(CP_ACP, 0, str, 2,
						reinterpret_cast<wchar_t*>(&this->value), 1);
				}
				// 2oCgڂȂ or ͈͊O
				else {
					this->start_ptr = NULL;
					this->current_pos = 0;
					this->eof_pos = 0;
					this->value = -1;
				}
			}
			// 1oCg
			else {
				// ASCIIR[h݊
				if (0x00 < first_byte && first_byte <= 0x7F) {
					this->value = first_byte;
				}
				// pJi
				else if (0xA1 <= first_byte && first_byte <= 0xDF) {
					// Unicodeϊ
					// TQ[g͍lĂȂ(SJIS̕ϊvH)
					char c = static_cast<char>(first_byte);
					::MultiByteToWideChar(CP_ACP, 0, &c, 1,
						reinterpret_cast<wchar_t*>(&this->value), 1);
				}
				// ͈͊O
				else {
					this->start_ptr = NULL;
					this->current_pos = 0;
					this->eof_pos = 0;
					this->value = -1;
				}
			}
		}
		// EOF
		else {
			this->start_ptr = NULL;
			this->current_pos = 0;
			this->eof_pos = 0;
			this->value = -1;
		}
		return *this;
	}
	// uCNgZq
	this_type operator++(int)
	{
		this_type tmp = *this;
		++*this;
		return tmp;
	}
};

//=======================================================================
// UTF16Iterator
//  z񂩂UTF-161ÂCe[^
//=======================================================================
class UTF16Iterator
	: public TextIteratorBase
{
private:
	// 2oCgǂݏoA|C^i߂
	int32_t get(void)
	{
		const int32_t temp =
			((this->current_pos <= this->eof_pos - 2)
			? *reinterpret_cast<const uint16_t*>(this->start_ptr + this->current_pos)
			: EOF);
		this->current_pos += 2;
		return temp;
	}

public:
	// ftHgRXgN^
	UTF16Iterator(void)
	{
	}
	// RXgN^
	UTF16Iterator(void *data_ptr, size_t file_size)
		: TextIteratorBase(data_ptr, file_size)
	{
		if (file_size > 0) {
			++*this;
			// BOMǂݔ΂
			if (this->value == 0xFEFF) {
				++*this;
			}
		} else {
			this->start_ptr = NULL;
		}
	}
	// OuCNgZq
	const this_type& operator++(void)
	{
		// 2oCgǂݏo
		const int32_t first_byte = this->get();
		if (first_byte != EOF && first_byte != 0x1A) {
			// Surrogateoꂽ
			if (IS_HIGH_SURROGATE(first_byte)) {
				// Surrogateǂݏo
				const int32_t second_byte = this->get();
				// Surrogate Pair
				if (second_byte != EOF && IS_LOW_SURROGATE(second_byte)) {
					this->value =
						(((first_byte & 0x03FF) << 10) | (second_byte & 0x03FF))
						+ 0x010000;
				}
				// 2oCgȂ or Surrogate͈̔͊O
				else {
					this->start_ptr = NULL;
					this->current_pos = 0;
					this->eof_pos = 0;
					this->value = -1;
				}
			}
			// SurrogateoĂȂ̂ɁASurrogateo(G[)
			else if (IS_LOW_SURROGATE(first_byte)) {
				this->start_ptr = NULL;
					this->current_pos = 0;
					this->eof_pos = 0;
					this->value = -1;
			}
			// UCS-2͈͓̔̕
			else {
				this->value = first_byte;
			}
		}
		// EOF
		else {
			this->start_ptr = NULL;
			this->current_pos = 0;
			this->eof_pos = 0;
			this->value = -1;
		}
		return *this;
	}
	// uCNgZq
	this_type operator++(int)
	{
		this_type tmp = *this;
		++*this;
		return tmp;
	}
};

//=======================================================================
// SetupFileParser
//  ZbgAbvt@C̉
//=======================================================================
class SetupFileParser
{
public:
	typedef std::map<std::wstring, SetupFileSection, LessNoCase> ParsedData;

	//
	// RXgN^
	// F
	//  data_ptr    - t@Cǂݍ񂾃obt@̃|C^
	//  data_size   - t@Cǂݍ񂾃obt@̃TCY
	//
	SetupFileParser(void *data_ptr, size_t data_size);
	//
	// parse
	//  ͂̊Jn
	// ߂lF
	//  EOF̈ʒu(EOFLꍇ̂)
	// F
	//  parsed_data_ptr - ͂f[^IuWFNg̃|C^
	//      Rs[ɎԂ邽߁A߂lɂȂB
	//      ֐ŕύX邱Ƃ𖾎邽߁Aă|C^nB
	//
	template <class T>
	FilePointerType parse(ParsedData *parsed_data_ptr);

private:
	enum ParseFuncIds {
		NORMAL,
		SECTION_NAME,
		COMMENT,
//        LINE_CONTINUATION,
		QUOTES,
		PARSE_FUNCS_COUNT
	};
	enum ParseState {
		LEADING_SPACES,
		FIELD,
		TRAILING_SPACES,
	};
	typedef void (SetupFileParser::*ParseFunc)(int32_t);

	ParseFuncIds current_func_id;
	ParseState state_id;

	bool is_line_start;
	// ̃ZNVɕĂꍇA
	// t@Cł̍s͈̔͂Ȃ̂h
	bool is_section_title_end;
	ParseFunc parse_funcs[PARSE_FUNCS_COUNT];

	// ͌ʂIuWFNg̃|C^
	ParsedData *sections;
	// ݂̃ZNVwCe[^
	// (Azz̃Ce[^͍폜܂ŗL)
	ParsedData::iterator it;
	// s̊Jnʒu
	FilePointerType line_start_point;

	// max size 512(NT5.x) or 4096(Vista) including '\0'
	//typedef
	//    std::basic_string<TCHAR, std::char_traits<TCHAR>, FixedAllocator<TCHAR, 512> > FixedString; 
	// ǂ݂ƂꎞIɕۑĂobt@
	std::wstring temp_buf;
	// ɑXy[XꎞIɕۑĂobt@
	std::wstring trailing_space_buf;

	// t@Cǂݍ񂾃obt@̃|C^
	void *data_ptr;
	// t@Cǂݍ񂾃obt@̃TCY
	size_t data_size;
	// ݂̃t@C|C^̈ʒu
	size_t current_pos;

	void addChar(int32_t c);
	void startSection(void);
	void endSection(void);
	void startLine(void);
	void endLine(void);
	void setKey(void);
	void addField(void);

	// ͊֐؂ւ
	void switchFunc(enum ParseFuncIds id);
	void switchState(enum ParseState id);
	// ͊֐
	void parseNormal(int32_t c);
	void parseSectionName(int32_t c);
	void parseComment(int32_t c);
	//void parseLineContinuation(int32_t c);
	void parseQuotes(int32_t c);
};

inline SetupFileParser::SetupFileParser(void *buf_ptr, size_t buf_size)
	: current_func_id(NORMAL)
	, state_id(LEADING_SPACES)
	, is_line_start(true)
	, is_section_title_end(false)
	, data_ptr(buf_ptr)
	, data_size(buf_size)
	, sections(NULL)
{
	parse_funcs[NORMAL] = &SetupFileParser::parseNormal;
	parse_funcs[SECTION_NAME] = &SetupFileParser::parseSectionName;
	parse_funcs[COMMENT] = &SetupFileParser::parseComment;
	//parse_funcs[LINE_CONTINUATION] = &SetupFileParser::parseLineContinuation;
	parse_funcs[QUOTES] = &SetupFileParser::parseQuotes;
}

template <class T>
inline FilePointerType SetupFileParser::parse(
	SetupFileParser::ParsedData *parsed_data_ptr)
{
	// ͌ʂIuWFNg̃|C^ݒ
	this->sections = parsed_data_ptr;
	// ݂̃ZNVwCe[^̏
	this->it = this->sections->end();
	// ݂̍s̊Jn_̏
	this->line_start_point = 0;
	T it(this->data_ptr, this->data_size);
	// end-of-streamCe[^
	const T it_end;
	// ͏̃C[v
	for (; it != it_end; ++it) {
		// ݂̃t@C|C^ʒu擾
		// (ZNVs̊Jn_EI_L^邽)
		this->current_pos = it.tellg();
		// Call Parse Function
		(this->*(this->parse_funcs[this->current_func_id]))(*it);
	}
	// Ō̍ssŏIĂȂ΍
	(this->*(this->parse_funcs[this->current_func_id]))('\n');
	// Ō̃ZNV̏I
	this->endSection();
	// EOF̈ʒuԂ
	return this->current_pos;
}

inline void SetupFileParser::addChar(int32_t c)
{
#ifdef UNICODE
	// Surrogate͍lĂȂ(SJIS̕ϊvH)
	this->temp_buf.push_back(static_cast<wchar_t>(c));
#else
	if (c > 0xFF) {
		this->temp_buf.push_back((c >> 8) & 0xFF);
	}
	this->temp_buf.push_back(c & 0xFF);
#endif
}

inline void SetupFileParser::startSection(void)
{
	// ̃ZNVf[^ǉ
	this->it = sections->find(this->temp_buf);
	if (this->it == this->sections->end()) {
		this->it = sections->insert(
			std::make_pair(this->temp_buf, SetupFileSection())).first;
	}
	// ݂̍s̊JnʒuZNV̊JnʒuƂ
	std::pair<FilePointerType, FilePointerType> range;
	range.first = this->line_start_point;
	this->it->second.ranges.push_back(range);
	// obt@NA
	this->temp_buf.clear();
}
inline void SetupFileParser::endSection(void)
{
	if (this->it != this->sections->end()) {
		// ݂̍s̊JnʒuZNV̏IʒuƂ
		this->it->second.ranges.back().second = this->line_start_point;
	}
}

inline void SetupFileParser::startLine(void)
{
	if (this->it == sections->end()) {
		//throw Win32Error(static_cast<DWORD>(HRESULT_FROM_SETUPAPI(ERROR_EXPECTED_SECTION_NAME)));
		throw MY_SETUP_API_EXCEPTION(HRESULT_FROM_SETUPAPI(ERROR_EXPECTED_SECTION_NAME));
	}
	SetupFileLine line(this->line_start_point);
	this->it->second.lines.push_back(line);
}
inline void SetupFileParser::endLine(void)
{
	this->line_start_point = this->current_pos;
	this->switchState(LEADING_SPACES);
	this->is_line_start = true;
	if (!this->is_section_title_end && it != this->sections->end() && !it->second.lines.empty()) {
		this->it->second.lines.back().setEndPosition(this->line_start_point);
	}
	this->is_section_title_end = false;
}
inline void SetupFileParser::setKey(void)
{
	this->it->second.lines.back().addField(0, this->temp_buf);
	this->temp_buf.clear();
}
inline void SetupFileParser::addField(void)
{
	if (it != this->sections->end() && !it->second.lines.empty()) {
		SetupFileLine& last_line =
			this->it->second.lines.back();
		unsigned field_count = last_line.getFieldCount();
		// tB[h+1
		++field_count;
		last_line.addField(field_count, this->temp_buf);
		// obt@̃NA
		this->temp_buf.clear();
	}
}

// ͊֐؂ւ
inline void SetupFileParser::switchFunc(enum ParseFuncIds id)
{
	this->current_func_id = id;
}
inline void SetupFileParser::switchState(enum ParseState id)
{
	if (this->state_id == TRAILING_SPACES) {
		this->trailing_space_buf.clear();
	}
	this->state_id = id;
}

inline void SetupFileParser::parseNormal(int32_t c)
{
	switch (c) {
	case '\n':
		if (this->state_id != LEADING_SPACES) {
			this->addField();
		}
		this->endLine();
		break;
	case '\"':
		if (this->is_line_start && this->state_id == LEADING_SPACES) {
			this->startLine();
		}
		if (this->state_id == TRAILING_SPACES) {
			this->temp_buf += this->trailing_space_buf;
		}
		this->switchState(FIELD);
		this->switchFunc(QUOTES);
		break;
	case ',':
		if (this->is_line_start && this->state_id == LEADING_SPACES) {
			this->startLine();
		}
		this->addField();
		this->is_line_start = false;
		this->switchState(LEADING_SPACES);
		break;
	case ';':
	case '#':
		if (this->state_id != LEADING_SPACES) {
			this->addField();
		}
		this->switchFunc(COMMENT);
		break;
	//case '\\':
	//    this->current_func_id = LINE_CONTINUATION;
	//    break;
	default:
		// 󔒋L̏(SpXy[X܂܂)
		// BMPO̕Ɩ肩?
		if (std::iswspace(static_cast<wint_t>(c))) {
			if (this->state_id == FIELD) {
				this->switchState(TRAILING_SPACES);
				this->trailing_space_buf.push_back(static_cast<wchar_t>(c));
			} else if (this->state_id == TRAILING_SPACES) {
				this->trailing_space_buf.push_back(static_cast<wchar_t>(c));
			}
		}
		// ZNV̊JnL
		else if (this->is_line_start
			&& this->state_id == LEADING_SPACES
			&& c == '[') {
				this->endSection();
				this->current_func_id = SECTION_NAME;
		}
		// L[
		else if (this->is_line_start && c == '=') {
			if (this->state_id == LEADING_SPACES) {
				this->startLine();
			} else {
				this->setKey();
			}
			this->is_line_start = false;
			this->switchState(LEADING_SPACES);
		}
		// ̑̕
		else {
			if (this->is_line_start && this->state_id == LEADING_SPACES) {
				this->startLine();
			} else if (this->state_id == TRAILING_SPACES) {
				this->temp_buf += this->trailing_space_buf;
			}
			this->switchState(FIELD);
			this->addChar(c);
		}
		break;
	}
}

inline void SetupFileParser::parseSectionName(int32_t c)
{
	switch (c) {
	case '\n':
		//throw SetupAPIError(HRESULT_FROM_SETUPAPI(ERROR_BAD_SECTION_NAME_LINE));
		throw MY_SETUP_API_EXCEPTION(HRESULT_FROM_SETUPAPI(ERROR_BAD_SECTION_NAME_LINE));
		break;
	case ']':
		// add section
		this->startSection();
		this->switchFunc(COMMENT);
		this->is_section_title_end = true;
		break;
	default:
		this->addChar(c);
		break;
	}
}

inline void SetupFileParser::parseComment(int32_t c)
{
	if (c == '\n') {
		this->endLine();
		this->switchFunc(NORMAL);
	}
}

//inline void SetupFileParser::parseLineContinuation(int32_t c)
//{
//}

inline void SetupFileParser::parseQuotes(int32_t c)
{
	switch (c) {
	// pɉsꍇAňpƂ݂Ȃ
	case '\n':
		this->addField();
		this->endLine();
	case '\"':
		this->switchFunc(NORMAL);
		break;
	default:
		this->addChar(c);
		break;
	}
}
};  // namespace MyLib

#endif // SETUP_FILE_PARSER_H__FAD9809C_AE84_4D3C_A4C3_0F775A801BD7__INCLUDED
