#ifndef SETUP_FILE_READER_H__FA827537_1202_4634_9A4B_10A6F365AB54__INCLUDED
#define SETUP_FILE_READER_H__FA827537_1202_4634_9A4B_10A6F365AB54__INCLUDED
// M$ compatible compilers support #pragma once
#if defined(_MSC_VER) && (_MSC_VER >= 1020)
#pragma once
#endif

#include "function.h"
#include "SetupFileParser.h"
#include "FileCompressionInfo.h"

/////////////////////////////////////////////////////////////////////////////
// SetupFileReader.h
// Classes in this file:
//  SetupFileLineIterator
//  SetupFileReader
/////////////////////////////////////////////////////////////////////////////

namespace MyLib {
class SetupFileLineIterator
	: public std::iterator<std::forward_iterator_tag, SetupFileLine>
{
public:
	typedef SetupFileLineIterator this_type;
	typedef SetupFileParser::ParsedData::iterator SectionIterator;
	typedef SetupFileSection::SetupFileLineCollection::iterator LineIterator;

protected:
	// ̓f[^̃|C^
	SetupFileParser::ParsedData* data_ptr;
	// ZNṼCe[^
	SectionIterator section_it;
	// s̃Ce[^
	LineIterator line_it;

public:
	// ftHgRXgN^ŐꂽCX^XI[Ƃ
	SetupFileLineIterator()
		: data_ptr(NULL)
	{
	}
	// RXgN^
	SetupFileLineIterator(
		SetupFileParser::ParsedData& parsed_data,
		const std::wstring& section)
		: data_ptr(&parsed_data)
		, section_it(data_ptr->find(section))
	{
		if (this->section_it != this->data_ptr->end() && !this->section_it->second.lines.empty()) {
			// ̓f[^̍ŏ̍sɈړ
			this->line_it = this->section_it->second.lines.begin();
		} else {
			data_ptr = NULL;
		}
	}
	SectionIterator getSectionIterator() const
	{
		return this->section_it;
	}
	LineIterator getLineIterator() const
	{
		return this->line_it;
	}
	reference operator*()
	{
		return *this->line_it;
	}
	pointer operator->()
	{
		return &(*this->line_it);
	}
	const this_type& operator++()
	{
		// ̓f[^
		if (this->section_it != this->data_ptr->end()) {
			++this->line_it;
			// ̓f[^̏I[ɒB
			if (this->line_it == this->section_it->second.lines.end()) {
				// ̓f[^𖳌ɂ
				this->section_it = this->data_ptr->end();
				this->data_ptr = NULL;
			}
		}
		// f[^Ȃ
		else {
			this->data_ptr = NULL;
		}
		return *this;
	}
	bool operator==(const this_type& rhs) const
	{
		return this->data_ptr == rhs.data_ptr;
	}
	bool operator!=(const this_type& rhs) const
	{
		return !(*this == rhs);
	}
};

//=======================================================================
// SetupFileReader
//  ZbgAbvt@Cǂ
//=======================================================================
class SetupFileReader
{
public:
	// RXgN^
	SetupFileReader(boost::filesystem::wpath const &file_path);
	// fXgN^
	virtual ~SetupFileReader();
	// š
	SetupFileLineIterator findFirstLine(
		LPCTSTR section_name,
		LPCTSTR key_name = NULL);
	// š
	SetupFileLineIterator findNextMatchLine(
		SetupFileLineIterator position,
		LPCTSTR key_name);
	// L[̌
	SetupFileLineIterator findMatchLine(
		SetupFileLineIterator position,
		unsigned field_index,
		LPCTSTR field_string);

protected:
	HANDLE file_handle;
	size_t file_size;
	HANDLE mapping_handle;
	void *view_ptr;

	bool is_file_unicode;
	std::wstring original_file_name;
	std::wstring actual_path;
	std::wstring temp_file_name;
	// ͌
	SetupFileParser::ParsedData data;
	// EOF̈ʒu
	FilePointerType eof_position;

	void close();
};


inline SetupFileReader::SetupFileReader(boost::filesystem::wpath const &file_path)
	: original_file_name(file_path.string())
	, file_handle(INVALID_HANDLE_VALUE)
	, mapping_handle(NULL)
	, view_ptr(NULL)
{
	// t@CkĂ邩ׂ
	FileCompressionInfo compression_info(file_path);
	if (!compression_info.exists()) {
		throw MY_WIN32_FILE_EXCEPTION(file_path.string(), ERROR_FILE_NOT_FOUND);
	}
	// ۂ̃t@C擾
	this->actual_path = compression_info.getActualPath().string();
	// kԂ擾
	const bool is_compressed =
		(compression_info.getCompressionType() != FILE_COMPRESSION_NONE);
	// kĂꍇ͉𓀂
	if (is_compressed) {
		boost::filesystem::wpath const temp_file_name = createTemporaryFilePath();
		this->temp_file_name = compression_info.decompressOrCopyFile(temp_file_name).string();
	}
	// t@C}bsOIuWFNg
	const std::wstring file_path_str =
		(is_compressed ? this->temp_file_name : this->actual_path);
	this->file_handle = ::CreateFile(
		file_path_str.c_str(),
		GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL, NULL);
	if (this->file_handle == INVALID_HANDLE_VALUE) {
		this->close();
		throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(file_path_str);
	}
	this->file_size = ::GetFileSize(this->file_handle, NULL);
	if(this->file_size == 0xFFFFFFFF) {
		this->close();
		throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(file_path_str);
	}
	this->mapping_handle = ::CreateFileMapping(
		this->file_handle, NULL, PAGE_READONLY, 0, 0, NULL);
	if (this->mapping_handle == NULL) {
		this->close();
		throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(file_path_str);
	}
	//  
	this->view_ptr =
		::MapViewOfFile(this->mapping_handle, FILE_MAP_READ, 0, 0, 0);
	if (this->view_ptr == NULL) {
		this->close();
		//throw MY_WIN32_EXCEPTION_LAST_ERROR;
		throw MY_WIN32_FILE_EXCEPTION_LAST_ERROR(file_path_str);
	}
	// Unicode`FbN
	const uint16_t bom = *reinterpret_cast<uint16_t*>(this->view_ptr);
	SetupFileParser prs(this->view_ptr, this->file_size);
	if (bom == 0xFEFF) {
		// Unicode(UTF-16)
		this->eof_position = prs.parse<UTF16Iterator>(&this->data);
		this->is_file_unicode = true;
	} else {
		// ShiftJIS(CP932)
		this->eof_position = prs.parse<CP932Iterator>(&this->data);
		this->is_file_unicode = false;
	}
}

inline SetupFileReader::~SetupFileReader()
{
	this->close();
}

inline void SetupFileReader::close()
{
	if (this->view_ptr != NULL) {
		::UnmapViewOfFile(this->view_ptr);
		this->view_ptr = NULL;
	}
	if (this->mapping_handle != NULL) {
		::CloseHandle(this->mapping_handle);
		this->mapping_handle = NULL;
	}
	if (this->file_handle != INVALID_HANDLE_VALUE) {
		::CloseHandle(this->file_handle);
		this->file_handle = INVALID_HANDLE_VALUE;
	}
	// ̃t@CkĂꍇA𓀂čꂽꎞt@C폜
	if (!this->temp_file_name.empty()) {
		::DeleteFile(this->temp_file_name.c_str());
		this->temp_file_name.clear();
	}
}

inline SetupFileLineIterator SetupFileReader::findFirstLine(
	LPCTSTR section_name,
	LPCTSTR key_name)
{
	SetupFileLineIterator temp(this->data, /*this->addition,*/ section_name);
	if (key_name != NULL) {
		temp = this->findMatchLine(temp, 0, key_name);
	}
	return temp;
}

inline SetupFileLineIterator SetupFileReader::findNextMatchLine(
	SetupFileLineIterator position,
	LPCTSTR key_name)
{
	return this->findMatchLine(++position, 0, key_name);
}

inline SetupFileLineIterator SetupFileReader::findMatchLine(
	SetupFileLineIterator position,
	unsigned field_index,
	LPCTSTR key_name)
{
	SetupFileLineIterator it_temp = position;
	const SetupFileLineIterator it_end;
	for (; it_temp != it_end; ++it_temp) {
		if ((field_index == 0 && !it_temp->hasKey())
			|| field_index > it_temp->getFieldCount()) {
				continue;
		}
		if (compareStringNoCase(it_temp->getStringField(field_index), key_name) == 0) {
			break;
		}
	}
	return it_temp;
}
};	// namespace MyLib

#endif // SETUP_FILE_READER_H__FA827537_1202_4634_9A4B_10A6F365AB54__INCLUDED
