#pragma once

namespace MyLib {
//=======================================================================
// std::auto_ptr HANDLE
//=======================================================================
template <HANDLE INVALID>
class AutoHandle
{
public:
	/*explicit*/ AutoHandle(HANDLE h = INVALID) : h_(h) {}

	AutoHandle(AutoHandle& right) : h_(right.release()) {}

	~AutoHandle() { this->close(); }

	HANDLE get() const { return this->h_; }

	HANDLE release()
	{
		HANDLE temp = this->h_;
		this->h_ = INVALID;
		return temp;
	}

	void reset(HANDLE h = INVALID)
	{
		if (this->h_ != h) {
			this->close();
			this->h_ = h;
		}
	}

	AutoHandle& operator=(AutoHandle& right)
	{
		this->reset(right.release());
		return *this;
	};

	bool isInvalid() const { return this->h_ == INVALID; }

private:
	HANDLE h_;

	void close()
	{
		if (this->h_ != INVALID) {
			::CloseHandle(this->h_);
			this->h_ = INVALID;
		}
	}
};

typedef AutoHandle<INVALID_HANDLE_VALUE> AutoHandleI;
typedef AutoHandle<NULL> AutoHandleN;
};  // namespace MyLib
