#ifndef EXCEPTION_H__F4B68BC8_B313_4A2E_8408_006F4C8C9BED__INCLUDED
#define EXCEPTION_H__F4B68BC8_B313_4A2E_8408_006F4C8C9BED__INCLUDED
// M$ compatible compilers support #pragma once
#if defined(_MSC_VER) && (_MSC_VER >= 1020)
#pragma once
#endif

/////////////////////////////////////////////////////////////////////////////
// Exception.h
//  H20(2008)/06/28 쐬
// Classes in this file:
//	Exception
//	FileException
//	FileTransferException
//  Win32Exception
//	Win32FileException
//	Win32FileTransferException
//  StdCException
//	StdCFileException
//	StdCFileTransferException
//  SetupAPIException
//	SetupAPIFileException
//	SetupAPIFileTransferException
/////////////////////////////////////////////////////////////////////////////

namespace MyLib {
	/////////////////////////////////////////////////////////////////////////
	// ƎONX̊NX
	/////////////////////////////////////////////////////////////////////////

	//=======================================================================
	// Exception
	//	ONX̊NX
	//=======================================================================
	template <class CharType>
	class Exception
	{
	protected:
		int error_code_;
		std::basic_string<CharType> error_text_;
		const CharType *error_file_;
		int error_line_;

	public:
		Exception(int error_code, const std::basic_string<CharType>& error_text, const CharType *error_file = NULL, int error_line = 0)
			: error_code_(error_code)
			, error_text_(error_text)
			, error_file_(error_file)
			, error_line_(error_line)
		{
		}

		int getErrorCode() const
		{
			return this->error_code_;
		}

		std::basic_string<CharType> getErrorText() const
		{
			return this->error_text_;
		}

		const CharType *getErrorFile() const
		{
			return this->error_file_;
		}

		int getErrorLine() const
		{
			return this->error_line_;
		}
	};

	template <class CharType>
	class FileException :
		public Exception<CharType>
	{
	protected:
		std::basic_string<CharType> file_name_;

	public:
		FileException(
			int error_code,
			const std::basic_string<CharType>& error_text,
			const std::basic_string<CharType>& file_name,
			const CharType *error_file = NULL,
			int error_line = 0)
			: Exception(error_code, error_text, error_file, error_line)
			, file_name_(file_name)
		{
		}

		std::basic_string<CharType> getFileName() const
		{
			return this->file_name_;
		}
	};

	template <class CharType>
	class FileTransferException :
		public Exception<CharType>
	{
	protected:
		std::basic_string<CharType> src_file_name_;
		std::basic_string<CharType> dest_file_name_;

	public:
		FileTransferException(
			int error_code,
			const std::basic_string<CharType>& error_text,
			const std::basic_string<CharType>& src_file_name,
			const std::basic_string<CharType>& dest_file_name,
			const CharType *error_file = NULL,
			int error_line = 0)
			: Exception(error_code, error_text, error_file, error_line)
			, src_file_name_(src_file_name)
			, dest_file_name_(dest_file_name)
		{
		}

		std::basic_string<CharType> getSourceFileName() const
		{
			return this->src_file_name_;
		}

		std::basic_string<CharType> getDestinationFileName() const
		{
			return this->dest_file_name_;
		}
	};

	/////////////////////////////////////////////////////////////////////////
	// Win32 ONX
	/////////////////////////////////////////////////////////////////////////

	// ̃G[bZ[WԂ
	inline const std::wstring makeWin32ErrorText(int error_code)
	{
		HLOCAL buf_ptr;
		uint32_t ret = ::FormatMessageW(
			FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
			NULL,
			error_code,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			reinterpret_cast<wchar_t*>(&buf_ptr),
			0,
			NULL);
		std::wstring result;
		if (ret != 0 && buf_ptr != NULL) {
			result = static_cast<const wchar_t*>(::LocalLock(buf_ptr));
			::LocalFree(buf_ptr);
		}
		return result;
	}

	class Win32Exception :
		public Exception<wchar_t>
	{
	public:
		Win32Exception(int error_code = ::GetLastError())
			: Exception(error_code, makeWin32ErrorText(error_code))
		{
		}

		Win32Exception(const wchar_t *error_file, int error_line, int error_code = ::GetLastError())
			: Exception(error_code, makeWin32ErrorText(error_code), error_file, error_line)
		{
		}
	};

	class Win32FileException :
		public FileException<wchar_t>
	{
	public:
		Win32FileException(
			const std::wstring& file_name,
			int error_code = ::GetLastError())
			: FileException(error_code, makeWin32ErrorText(error_code), file_name)
		{
		}

		Win32FileException(
			const wchar_t *error_file,
			int error_line,
			const std::wstring& file_name,
			int error_code = ::GetLastError())
			: FileException(error_code, makeWin32ErrorText(error_code), file_name, error_file, error_line)
		{
		}
	};

	class Win32FileTransferException :
		public FileTransferException<wchar_t>
	{
	public:
		Win32FileTransferException(
			const std::wstring& src_file_name,
			const std::wstring& dest_file_name,
			int error_code = ::GetLastError())
			: FileTransferException(error_code, makeWin32ErrorText(error_code), src_file_name, dest_file_name)
		{
		}

		Win32FileTransferException(
			const wchar_t *error_file,
			int error_line,
			const std::wstring& src_file_name,
			const std::wstring& dest_file_name,
			int error_code = ::GetLastError())
			: FileTransferException(error_code, makeWin32ErrorText(error_code), src_file_name, dest_file_name, error_file, error_line)
		{
		}
	};

	/////////////////////////////////////////////////////////////////////////
	// WCCuONX
	/////////////////////////////////////////////////////////////////////////

    inline const std::wstring makeStdCErrorText(int errno_value)
    {
#if defined(_MSC_VER) && (_MSC_VER >= 1400) // VC 2005
        char err_text[MAX_PATH];
        err_text[0] = '\0';
        ::strerror_s(err_text, _countof(err_text), errno_value);
#else // others
		const char *err_text = std::strerror(errno_value);
#endif
        std::wstring result;
		std::copy(err_text, err_text + std::strlen(err_text), std::back_inserter(result));
        return result;
    }

	class StdCException :
		public Exception<wchar_t>
	{
	public:
		StdCException(int error_code)
			: Exception(error_code, makeStdCErrorText(error_code))
		{
		}

		StdCException(
			const wchar_t *error_file,
			int error_line,
			int error_code = ::GetLastError())
			: Exception(error_code, makeStdCErrorText(error_code), error_file, error_line)
		{
		}
	};

	class StdCFileException :
		public FileException<wchar_t>
	{
	public:
		StdCFileException(
			const std::wstring& file_name,
			int error_code = ::GetLastError())
			: FileException(error_code, makeStdCErrorText(error_code), file_name)
		{
		}

		StdCFileException(
			const wchar_t *error_file,
			int error_line,
			const std::wstring& file_name,
			int error_code = ::GetLastError())
			: FileException(error_code, makeStdCErrorText(error_code), file_name, error_file, error_line)
		{
		}
	};

	class StdCFileTransferException :
		public FileTransferException<wchar_t>
	{
	public:
		StdCFileTransferException(
			const std::wstring& src_file_name,
			const std::wstring& dest_file_name,
			int error_code = ::GetLastError())
			: FileTransferException(error_code, makeStdCErrorText(error_code), src_file_name, dest_file_name)
		{
		}

		StdCFileTransferException(
			const wchar_t *error_file,
			int error_line,
			const std::wstring& src_file_name,
			const std::wstring& dest_file_name,
			int error_code = ::GetLastError())
			: FileTransferException(error_code, makeStdCErrorText(error_code), src_file_name, dest_file_name, error_file, error_line)
		{
		}
	};

	/////////////////////////////////////////////////////////////////////////
	// Setup API ONX
	/////////////////////////////////////////////////////////////////////////

	// Setup APIpGetLastError()֐
	inline HRESULT getSetupAPILastError()
	{
		DWORD error_code = ::GetLastError();
		return HRESULT_FROM_SETUPAPI(error_code);
	}

	class SetupAPIException :
		public Win32Exception
	{
	public:
		SetupAPIException(int error_code = getSetupAPILastError())
			: Win32Exception(error_code)
		{
		}

		SetupAPIException(const wchar_t *error_file, int error_line, int error_code = getSetupAPILastError())
			: Win32Exception(error_file, error_line, error_code)
		{
		}
	};

	class SetupAPIFileException :
		public Win32FileException
	{
	public:
		SetupAPIFileException(
			const std::wstring& file_name,
			int error_code = getSetupAPILastError())
			: Win32FileException(file_name, error_code)
		{
		}

		SetupAPIFileException(
			const wchar_t *error_file,
			int error_line,
			const std::wstring& file_name,
			int error_code = getSetupAPILastError())
			: Win32FileException(error_file, error_line, file_name, error_code)
		{
		}
	};

	class SetupAPIFileTransferException :
		public Win32FileTransferException
	{
	public:
		SetupAPIFileTransferException(
			const std::wstring& src_file_name,
			const std::wstring& dest_file_name,
			int error_code = ::GetLastError())
			: Win32FileTransferException(src_file_name, dest_file_name, error_code)
		{
		}

		SetupAPIFileTransferException(
			const wchar_t *error_file ,
			int error_line,
			const std::wstring& src_file_name,
			const std::wstring& dest_file_name,
			int error_code = ::GetLastError())
			: Win32FileTransferException(error_file, error_line, src_file_name, dest_file_name, error_code)
		{
		}
	};
};

#endif // EXCEPTION_H__F4B68BC8_B313_4A2E_8408_006F4C8C9BED__INCLUDED
