﻿#ifndef INI_FILE_H__B0DA9D9C_827E_4103_A701_32F2DF60D52B__INCLUDED
#define INI_FILE_H__B0DA9D9C_827E_4103_A701_32F2DF60D52B__INCLUDED
// M$ compatible compilers support #pragma once
#if defined(_MSC_VER) && (_MSC_VER >= 1020)
#pragma once
#endif

/**
 * INI file manipulation class
 */
class IniFile
{
	std::wstring profile_name;

public:
	/**
	 * constructor
	 * @param file_name
	 */
	IniFile(boost::filesystem::wpath const &file_name)
		: profile_name(file_name.string())
	{
	}

	/////////////////////////////////////////////////////////////////////////

	/**
	 * Returns a String value from INI file
	 * @param  section
	 * @param  key
	 * @param  default_value
	 * @return string
	 */
	std::wstring getString(std::wstring const &section, std::wstring const &key, std::wstring const &default_value)
	{
		std::vector<TCHAR> tmp_buf; // APIから受け取る文字列を格納するバッファ
		                            // (std::stringの内部メモリは連続性が規格で保証されていない)
		DWORD buf_size = 32;        // バッファのサイズ(TCHAR単位)
		DWORD ret;                  // バッファに書き込まれた文字数（終端の NULL 文字を除く）

		do {
			buf_size <<= 1;
			tmp_buf.resize(buf_size, L'\0'); // バッファの確保
			ret = ::GetPrivateProfileString(section.c_str(), key.c_str(), default_value.c_str(), &tmp_buf.at(0),
				buf_size, this->profile_name.c_str());
		} while (ret >= buf_size - 1);  // 書き込まれた文字数がバッファサイズぎりぎりのとき
		                                // バッファが足りないと見なす。
		return &tmp_buf.at(0);
	}

	/**
	 * Returns an Integer value from INI file
	 * @param  section
	 * @param  key
	 * @param  default_value
	 * @return integer
	 */
	unsigned int getInt(std::wstring const &section, std::wstring const &key, unsigned int default_value)
	{
		return ::GetPrivateProfileInt(section.c_str(), key.c_str(),
			default_value, this->profile_name.c_str());
	}

	/**
	 * Returns a Float value from INI file
	 * @param  section
	 * @param  key
	 * @param  default_value;
	 * @return float
	 */
	double getFloat(std::wstring const &section, std::wstring const &key, double default_value)
	{
		double result = default_value;
		std::wstring str = getString(section, key, _T(""));
		if (str.length() > 0) {
			result = _tstof(str.c_str());
		}
		return result;
	}

	/////////////////////////////////////////////////////////////////////////

	/**
	 * Writes a String value to the INI file
	 * @param  section
	 * @param  key
	 * @param  value
	 */
	void setString(std::wstring const &section, std::wstring const &key, std::wstring const &value)
	{
		::WritePrivateProfileString(section.c_str(), key.c_str(),
			value.c_str(), this->profile_name.c_str());
	}

	/**
	 * Writes an Integer value to the INI file
	 * @param  section
	 * @param  key
	 * @param  value
	 */
	void setInt(std::wstring const &section, std::wstring const &key, unsigned int value)
	{
		std::wstringstream ss;
		ss << value;
		this->setString(section, key, ss.str());
	}

	/**
	 * Writes a Float value to the INI file
	 * @param  section
	 * @param  key
	 * @param  value;
	 */
	void setFloat(std::wstring const &section, std::wstring const &key, double value)
	{
		std::wstringstream ss;
		ss << value;
		this->setString(section, key, ss.str());
	}

	/**
	 * Deletes a Section from INI file
	 * @param  section
	 */
	void deleteSection(std::wstring const &section)
	{
		::WritePrivateProfileString(section.c_str(),
			NULL, NULL, this->profile_name.c_str());
	}

	/**
	 * Deletes a Key from INI file
	 * @param  section
	 * @param  key
	 */
	void deleteKey(std::wstring const &section, std::wstring const &key)
	{
		::WritePrivateProfileString(section.c_str(),
			key.c_str(), NULL, this->profile_name.c_str());
	}
};

#endif  // INI_FILE_H__B0DA9D9C_827E_4103_A701_32F2DF60D52B__INCLUDED
