﻿#ifndef NUMERIC_EDIT_H__08FFB2E9_98DC_473B_9A6B_4358004A0725__INCLUDED
#define NUMERIC_EDIT_H__08FFB2E9_98DC_473B_9A6B_4358004A0725__INCLUDED
// MS compatible compilers support #pragma once
#if defined(_MSC_VER) && (_MSC_VER >= 1020 )
#pragma once
#endif

/////////////////////////////////////////////////////////////////////////////
// CNumericEdit
//  数値のみを入力できるテキストボックス
//  リソースプロパティの「Number」では
//	マイナス記号や小数点も入力できないため
//  H19(2007)/02/14 作成
//  H19(2007)/12/13 cppファイルを統合
//	H19(2007)/12/14 二重include防止処理を変更
//	H20(2008)/12/05 小数・負数の許可・禁止を変更できるようにした
//	                Ctrl+V などのショートカットキーが使えなかったのを修正
//	                フォーカスを持つときは IME を無効にするようにした
/////////////////////////////////////////////////////////////////////////////

#pragma comment(lib, "imm32.lib")

class CNumericEdit : public CWindowImpl<CNumericEdit, CEdit>
{
	bool	m_bIsFloat;		//true:小数点数 / false:整数
	bool	m_bIsSigned;	//true:符号あり / false:符号なし
	//bool	m_bIsOnKeyDown;	//true:キーが押された / false:キーが押されなかった
	//CString	m_strPrevText;
	HIMC h_imc_;

public:
	DECLARE_WND_SUPERCLASS(NULL, CEdit::GetWndClassName());

    // コンストラクション
	CNumericEdit(bool uses_signed = true, bool uses_float = true) :
		m_bIsSigned(uses_signed),
		m_bIsFloat(uses_float),
		h_imc_(NULL)
		//m_bIsOnKeyDown(false)
	{
	}

	// 負数の許可
	void enableSigned() { this->m_bIsSigned = true; }
	// 小数の許可
	void enableFloat() { this->m_bIsFloat = true; }
	// 負数の禁止
	void disableSigned() { this->m_bIsSigned = false; }
	// 小数の禁止
	void disableFloat() { this->m_bIsFloat = false; }

    // メッセージ マップ
	BEGIN_MSG_MAP_EX(CNumericEdit)
		MSG_WM_CHAR(OnChar)
		MSG_WM_PASTE(OnPaste)
        //MESSAGE_HANDLER(WM_KEYDOWN, OnKeyDown)
        //MESSAGE_HANDLER(WM_KEYUP, OnKeyUp)
		//REFLECTED_COMMAND_CODE_HANDLER(EN_CHANGE, OnChange)
		REFLECTED_COMMAND_CODE_HANDLER_EX(EN_SETFOCUS, OnSetFocus)
		REFLECTED_COMMAND_CODE_HANDLER_EX(EN_KILLFOCUS, OnKillFocus)
		DEFAULT_REFLECTION_HANDLER()
    END_MSG_MAP_EX()

// メッセージ マップ関数
// Handler prototypes (uncomment arguments if needed):
//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	void OnChar(TCHAR nChar, unsigned int nRepCnt, unsigned int nFlags);
	void OnPaste();
    //LRESULT OnKeyDown(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
    //LRESULT OnKeyUp(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	//LRESULT OnChange(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnSetFocus(unsigned int uNotifyCode, int nID, HWND wndCtl);
	LRESULT OnKillFocus(unsigned int uNotifyCode, int nID, HWND wndCtl);
};

inline void CNumericEdit::OnChar(TCHAR tChar, unsigned int /*nRepCnt*/, unsigned int /*nFlags*/)
{
	CString text0;
	GetWindowText(text0);

	CString text;
	int nSelStart, nSelEnd;
	GetSel(nSelStart, nSelEnd);
	if (nSelStart == nSelEnd) {//テキストが選択されていない。
		text = text0;//文字列データ（=表示されているテキスト）
	} else {//テキストの一部または全部が選択されている。
		for (int i = 0; i < text0.GetLength(); ++i) {
			if (i < nSelStart || nSelEnd <= i) {
				text += text0.GetAt(i);//表示されているテキストから選択部分を除いたもの
			}
		}
	}

	//数字、小数点（１個のみ）、マイナス記号（１個のみ）、バックスペースだけを受け付ける。
	if (
		(::IsCharAlphaNumeric(tChar) && !::IsCharAlpha(tChar))   //数字
		|| (tChar == _T('.') && m_bIsFloat  && text.Find(_T('.')) == -1) //唯一の小数点
		|| (tChar == _T('-') && nSelStart == 0 && m_bIsSigned && text.Find(_T('-')) == -1) //唯一のマイナス記号
		|| (0x01 <= tChar && tChar <= 0x1A)//(tChar == _T('\b')) //Back space
	  ) {
		DefWindowProc();
	}
}

inline void CNumericEdit::OnPaste()
{
	// 現在入力されている文字列を取得
	CString text0;
	GetWindowText(text0);
	// キャレットの位置・選択範囲を取得
	int nSelStart, nSelEnd;
	GetSel(nSelStart, nSelEnd);
	// 貼り付け処理が行われた場合の文字列を作成
	CString text;
	// キャレットより前
	for (int i = 0; i < nSelStart; ++i) {
		text += text0.GetAt(i);
	}
	// クリップボードにテキストデータが入っているかを調べる
	if( ::IsClipboardFormatAvailable(CF_TEXT) ) {
		// クリップボードのオープン
		if( ::OpenClipboard(NULL) ) {
			CString clipboard_text;
			// クリップボードからデータを取得し、strTextへコピー
			HANDLE hMem = ::GetClipboardData(CF_UNICODETEXT);
			LPTSTR pMem = (LPTSTR)::GlobalLock(hMem);
			LPTSTR p = clipboard_text.GetBufferSetLength(::lstrlen(pMem));
			::lstrcpy(p, pMem);
			::GlobalUnlock(hMem);
			::CloseClipboard();
			text += clipboard_text;
		}
	}
	// キャレット以降
	for (int i = nSelEnd; i < text0.GetLength(); ++i) {
		text += text0.GetAt(i);//表示されているテキストから選択部分を除いたもの
	}

	//キー入力以外によって表示文字列が変更されたときの処理
	int dp = 0;  //小数点（ピリオド）の数
	for (int i = 0; i < text.GetLength(); ++i) {
		TCHAR tChar = text.GetAt(i);
		if(tChar == _T('.')){
			dp++;
		}
		if(
			(::IsCharAlphaNumeric(tChar) && !::IsCharAlpha(tChar)) //数字
			|| (m_bIsFloat  && dp == 1 && tChar == _T('.')) //唯一の小数点
			|| (m_bIsSigned && i  == 0 && tChar == _T('-')) //唯一かつ正しい位置のマイナス記号
			  ){
			; //何もしない。
		} else {
			MessageBox(
				_T("\"") + text + _T("\" は無効だよ～\n操作をキャンセルしちゃうよ。"),
				_T("ごめんっ！"),
				MB_OK | MB_ICONASTERISK
			);
			return;// 0;
		}
	}

	/*return*/ this->DefWindowProc();
}

//inline LRESULT CNumericEdit::OnKeyDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
//{
//	m_bIsOnKeyDown = true;
//    return DefWindowProc(uMsg, wParam, lParam);
//}
//
//inline LRESULT CNumericEdit::OnKeyUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
//{
//	m_bIsOnKeyDown = false;
//    return DefWindowProc(uMsg, wParam, lParam);
//}
//
//inline LRESULT CNumericEdit::OnChange(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//{
//	CString text;
//
//	GetWindowText(text);
//
//	//変更後の表示文字列が空のときの処理
//	if(text.IsEmpty()){
//		m_strPrevText.Empty(); //文字列データを空にする。
//		return 0;
//	}
//
//	//キー入力によって表示文字列が変更されたときの処理
//	//キーの種類はOnCharでチェック済みなので、マイナス記号の有無と位置をチェックする。
//	if(m_bIsOnKeyDown){
//		if(text.Find(_T('-')) == -1 || text.GetAt(0) == _T('-')){ //マイナス記号がない、あるいは位置が正しい
//			m_strPrevText = text;
//			//GetWindowText(m_strPrevText); //文字列データを更新する。
//		}
//		else{ //N.G.
//			SetWindowText(m_strPrevText); //文字列データ（=直前のテキスト）を表示する。
//			//::MessageBeep(MB_OK);
//		}
//		return 0;
//	}
//
//	////キー入力以外によって表示文字列が変更されたときの処理
//	//TCHAR tChar;
//	//int i;
//	//int dp = 0; //小数点（ピリオド）の数
//
//	//for(i=0; i<text.GetLength(); i++){
//	//	tChar = text.GetAt(i);
//	//	if(tChar == _T('.')){
//	//		dp++;
//	//	}
//
//	//	if(
//	//		(::IsCharAlphaNumeric(tChar) && !::IsCharAlpha(tChar)) //数字
//	//		|| (m_bIsFloat  && dp == 1 && tChar == _T('.')) //唯一の小数点
//	//		|| (m_bIsSigned && i  == 0 && tChar == _T('-')) //唯一かつ正しい位置のマイナス記号
//	//		  ){
//	//		; //何もしない。
//	//	}
//	//	else{
//	//		SetWindowText(m_strPrevText); //文字列データ（=直前のテキスト）を表示する
//	//		MessageBox(
//	//			_T("\"") + text + _T("\" は無効だよ～\n操作をキャンセルしちゃうよ。"),
//	//			_T("ごめんっ！"),
//	//			MB_OK | MB_ICONASTERISK
//	//		);
//	//		return 0;
//	//	}
//	//}
//
//	GetWindowText(m_strPrevText);
//
//    return 0;
//}

inline LRESULT CNumericEdit::OnSetFocus(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	// IME をオフ
	this->h_imc_ = ::ImmAssociateContext(this->m_hWnd, NULL);

	return 0;
}

inline LRESULT CNumericEdit::OnKillFocus(unsigned int /*uNotifyCode*/, int /*nID*/, HWND /*wndCtl*/)
{
	// IME をオン
	::ImmAssociateContext(this->m_hWnd, this->h_imc_);


	CString strZero = m_bIsFloat ? _T("0.0") : _T("0");
	CString text;
	GetWindowText(text);

	if(text.IsEmpty() || text == _T("-") || text == _T(".")){
		SetWindowText(strZero); //"0.0" or "0"
		return 0;
	}

	CString next_text = text;

	bool bIsMinus = false;
	if (next_text.GetAt(0) == '-') {
		next_text = next_text.Right(next_text.GetLength() - 1); //"-nnn"-->"nnn"
		bIsMinus = true;
	}

	while (next_text.GetAt(0) == _T('0')) { //"000nnn"-->"nnn"
		next_text = next_text.Right(next_text.GetLength() - 1);
		if (next_text.IsEmpty()) { // "0"
			if (this->m_bIsFloat) {
				SetWindowText(strZero); //"0"-->"0.0"
			}
			return 0;
		}
	}

	if (m_bIsFloat) {
		if (next_text.Find('.') != -1) { //a decimal point exists
			while (next_text.Right(1) == _T("0")) { //"n.nnn000"-->"n.nnn"
				next_text = next_text.Left(next_text.GetLength() - 1);
			}
			if (next_text.Right(1) == _T(".")) {
				next_text += _T('0'); //"nnn."-->"nnn.0"
			}
			if (next_text.GetAt(0) == _T('.')) {
				next_text = _T('0') + next_text; //".nnn"-->"0.nnn"
			}
		} else {
			next_text += _T(".0"); //"nnn"-->"nnn.0"
		}
	}

	if (bIsMinus && next_text != _T("0.0") && next_text != _T("0")) {
		next_text = _T('-') + next_text; //"nnn"-->"-nnn"
	}

	if (text != next_text) {
		SetWindowText(next_text);
	}
	return 0;
}

#endif	// NUMERIC_EDIT_H__08FFB2E9_98DC_473B_9A6B_4358004A0725__INCLUDED
